/***************************************************************************
 *   Copyright (C) 2006 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "transkode.h"
#include "transkodelistviewitem.h"

#include <qlayout.h>
#include <qcursor.h>
#include <qtooltip.h>
#include <qevent.h>

#include <kapplication.h>
#include <kglobal.h>
#include <kstandarddirs.h>
#include <klocale.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kshortcut.h>
#include <kstdaccel.h>
#include <kstatusbar.h>
#include <kconfig.h>
#include <kurl.h>
#include <kmessagebox.h>
#include <kpopupmenu.h>
#include <klistviewsearchline.h>

TransKode::TransKode():
	KMainWindow( 0, "TransKode" ),

	m_logger( QString::null ),
	m_config( ::locate( "appdata", "defaultsrc" ) ),
	m_configDialog( ::locate( "appdata", "defaultsrc" ), false, this ),
	m_worker( m_logger, m_config ),

	m_listView( new TransKodeListView( this, m_config, m_worker ) ),
	m_systemTray( new KSystemTray( this ) )
{
	//setState( idle );
	updateState();

	setCentralWidget( m_listView ); // tell the KMainWindow that this is indeed the main widget
	setupActions(); // then, setup our actions
	statusBar()->show(); // a status bar
	toolBar()->show(); // and a tool bar

	setupGUI();

	// For some strange reason this #1) has to be plugged into something
	// for it to work and #2) has to be plugged after setupGUI execution
	m_searchManagerAction->plug( new KPopupMenu( this ) );

	m_config.load( ::locate( "appdata", "transkoderc" ) );
	m_configDialog.setConfig( m_config );

	m_logger.setFile( m_config.generalOpt( GeneralOpt::log ), true );
	m_logger.setMsgPrefix( "[app] " );

	m_systemTray->setPixmap( KSystemTray::loadIcon( "transkode" ) );
	if ( m_config.generalOpt( GeneralOpt::show_tray ) == "true" )
		m_systemTray->setShown( true );

	connect( m_systemTray, SIGNAL( quitSelected() ), this, SLOT( slotConfirmQuit() ) );
	connect( &m_configDialog, SIGNAL( configSaved() ), this, SLOT( slotUpdateConfig() ) ) ;

	connect( &m_worker, SIGNAL( mtRoundStarted(long) ), this, SLOT( slotRoundStarted() ) ) ;
	connect( &m_worker, SIGNAL( mtRoundFinished(long) ), this, SLOT( slotRoundFinished() ) );

	connect( m_listView, SIGNAL( jobFinished() ), this, SLOT( slotJobFinished() ) ) ;
	connect( m_listView, SIGNAL( contentAdded() ), this, SLOT( slotAdded() ) ) ;
	connect( m_listView, SIGNAL( allCleared() ), this, SLOT( slotAllCleared() ) ) ;
	connect( m_listView, SIGNAL( finishedCleared() ), this, SLOT( slotFinishedCleared() ) ) ;

	connect( m_listView, SIGNAL( message(const QString&) ), statusBar(), SLOT( message(const QString&) ) );

	connect( m_listView, SIGNAL( selectionChanged() ), this, SLOT( slotSetActionsState() ) );

	LOG( "TransKode: starting worker", APP );
	m_worker.start();

	loadConfig();
}

TransKode::~TransKode()
{
	LOG( "~TransKode: stopping worker", APP );
	m_worker.resume();
	m_worker.stop();

	LOG( "~TransKode: waiting worker to end", APP );
	m_worker.wait();

	LOG( "~TransKode: worker ended", APP );
}

void TransKode::setupActions()
{
	KStdAction::quit( this, SLOT( slotConfirmQuit() ), actionCollection() );
	KStdAction::preferences( this, SLOT( slotShowPreferences() ), actionCollection() );

	KAction* trayConfigure = new KAction(
		i18n( "&Configure..." ),
 		"configure",					// Action icon
		KShortcut( "" ),				// Action accelerator
		this,							// Action receiver
		SLOT( slotShowPreferences() ),	// Action slot
		actionCollection(),				// Action collection manager
		"tray_configure" );				// Action name/identifier
	trayConfigure->plug( m_systemTray->contextMenu(), 1 );

	// Create a toolBar named 'searchToolBar' and add a search widget to it
	m_searchAction = new KWidgetAction(
		new KListViewSearchLineWidget( m_listView, toolBar( "searchToolBar" ) ),
		i18n( "Search" ),				// Action text
 		"location_edit",				// Action icon
		this,							// Action receiver
		SLOT( slotShowPreferences() ),		// Action slot // TODO wtf is this doing?
		actionCollection(),				// Action collection manager
		"search" );						// Action name/identifier
	m_searchAction->plug( toolBar( "searchToolBar" ) );

	// Create an action asociated to the toolBar created above so we can enable/disable the filter
	m_searchManagerAction = new KToggleToolBarAction(
		"searchToolBar",
		"Toggle Search",
		actionCollection(),
		"search_manager" );
	connect( m_searchManagerAction, SIGNAL( toggled(bool) ), this, SLOT( slotToggleFilterToolBar(bool) ) );

	new KAction(
		i18n( "&Add Files" ),			// Action text
 		"edit_add",						// Action icon
		KShortcut( "Insert" ),			// Action accelerator
		m_listView,						// Action receiver
		SLOT( addFiles() ),				// Action slot
		actionCollection(),				// Action collection manager
		"add_files" );					// Action name/identifier

	new KAction(
		i18n( "&Remove Files" ),			// Action text
 		"edit_remove",					// Action icon
		KShortcut( "Delete" ),			// Action accelerator
		m_listView,						// Action receiver
		SLOT( removeFiles() ),			// Action slot
		actionCollection(),				// Action collection manager
		"remove_files" );				// Action name/identifier

	m_selectAllAction = new KAction(
		i18n( "&Select All" ),			// Action text
 		"player_playlist",				// Action icon
		KStdAccel::SelectAll,			// Action accelerator
		m_listView,						// Action receiver
		SLOT( selectAll() ),			// Action slot
		actionCollection(),				// Action collection manager
		"select_all" );					// Action name/identifier
	m_selectAllAction->setEnabled( false );

	m_invertSelectionAction = new KAction(
		i18n( "&Invert Selection" ),		// Action text
		KShortcut( "Ctrl+I" ),			// Action accelerator
		m_listView,						// Action receiver
		SLOT( invertSelection() ),		// Action slot
		actionCollection(),				// Action collection manager
		"invert_selection" );			// Action name/identifier
	m_invertSelectionAction->setEnabled( false );

	m_clearAllAction = new KAction(
		i18n( "&Clear" ),				// Action text
		"view_remove",					// Action icon
		KShortcut( "Ctrl+Delete" ),		// Action accelerator
		m_listView,						// Action receiver
		SLOT( clearAll() ),				// Action slot
		actionCollection(),				// Action collection manager
		"clear" );						// Action name/identifier
	m_clearAllAction->setEnabled( false );

	m_clearFinishedAction = new KAction(
		i18n( "Clear &Finished" ),		// Action text
		"button_ok",					// Action icon
		KShortcut( "Ctrl+C" ),			// Action accelerator
		m_listView,						// Action receiver
		SLOT( clearFinished() ),		// Action slot
		actionCollection(),				// Action collection manager
		"clear_finished" );				// Action name/identifier
	m_clearFinishedAction->setEnabled( false );

	m_refreshFilesAction = new KAction(
		i18n( "&Refresh" ),				// Action text
		"reload"	,					// Action icon
		KShortcut( "Ctrl+R" ),			// Action accelerator
		m_listView,						// Action receiver
		SLOT( refresh() ),				// Action slot
		actionCollection(),				// Action collection manager
		"refresh" );					// Action name/identifier
	m_refreshFilesAction->setEnabled( false );

	m_enqueueFilesAction = new KAction(
		i18n( "En&queue Files" ),		// Action text
 		"player_play",					// Action icon
		KShortcut( "Ctrl+Q" ),			// Action accelerator
		m_listView,						// Action receiver
		SLOT( queueFiles() ),			// Action slot
		actionCollection(),				// Action collection manager
		"queue_files" );				// Action name/identifier
	m_enqueueFilesAction->setEnabled( false );

	m_dequeueFilesAction = new KAction(
		i18n( "&Dequeue Files" ),		// Action text
 		"player_stop",					// Action icon
		KShortcut( "Ctrl+D" ),			// Action accelerator
		m_listView,						// Action receiver
		SLOT( dequeueFiles() ),			// Action slot
		actionCollection(),				// Action collection manager
		"dequeue_files" );				// Action name/identifier
	m_dequeueFilesAction->setEnabled( false );

	m_pauseEncodingAction = new KToggleAction(
		i18n( "&Pause" ),					// Action text
		"player_pause",					// Action icon
		KShortcut( "Ctrl+P" ),			// Action accelerator
		this,							// Action receiver
		SLOT( slotTogglePaused() ),		// Action slot
		actionCollection(),				// Action collection manager
		"pause_encoding" );				// Action name/identifier
	m_pauseEncodingAction->plug( m_systemTray->contextMenu(), 1 );
	m_pauseEncodingAction->setEnabled( false );
}

void TransKode::slotRoundStarted()
{
	//setState( transcoding );
	updateState();
	m_pauseEncodingAction->setEnabled( true );
}

void TransKode::slotRoundFinished()
{
	//setState( idle );
	updateState();
	m_pauseEncodingAction->setEnabled( false );
}

void TransKode::slotJobFinished()
{
	updateState();
	m_clearFinishedAction->setEnabled( true );
	slotSetActionsState();
}

void TransKode::slotAdded()
{
	m_clearAllAction->setEnabled( true );
	m_selectAllAction->setEnabled( true );
	m_invertSelectionAction->setEnabled( true );
}

void TransKode::slotAllCleared()
{
	m_clearAllAction->setEnabled( false );
	m_clearFinishedAction->setEnabled( false );
	m_selectAllAction->setEnabled( false );
	m_invertSelectionAction->setEnabled( false );

	m_pauseEncodingAction->setEnabled( false );
	m_enqueueFilesAction->setEnabled( false );
	m_dequeueFilesAction->setEnabled( false );
}

void TransKode::slotFinishedCleared()
{
	m_clearFinishedAction->setEnabled( false );

	slotSetActionsState();
}

void TransKode::slotTogglePaused()
{
	if ( m_worker.paused() )
	{
		//setState( transcoding );
		m_worker.resume();
	}
	else
	{
		//setState( paused );
		m_worker.pause();
	}
	updateState();
}

void TransKode::slotSetActionsState()
{
// 	startenqueue tiene que haber seleccionados items enqueuables
// 	stopdequeue tiene que haber seleccionado items queuables
// 	refresh tiene que haber seleccionado items con profile pero no encolados

	bool queueableItems = false;
	bool enqueuedItems = false;
	bool refresheableItems = false;

	TransKodeListViewItem* item = 0;
	for ( QListViewItemIterator it( m_listView, QListViewItemIterator::Selected ) ; it.current() != 0; ++it )
	{
		if ( it.current()->rtti() != TRANSKODE )
			continue;

		item = (TransKodeListViewItem*)it.current();

		if ( item->isEnqueued() ) enqueuedItems = true;
		if ( item->isQueueable() ) queueableItems = true;
		if ( item->isRefresheable() ) refresheableItems = true;

		if ( enqueuedItems && queueableItems && refresheableItems )
			break;
	}

	m_enqueueFilesAction->setEnabled( queueableItems );
	m_dequeueFilesAction->setEnabled( enqueuedItems );
	m_refreshFilesAction->setEnabled( refresheableItems );
}


void TransKode::slotShowPreferences()
{
	QSize size = m_configDialog.size();
	m_configDialog.setInitialSize( size );
	m_configDialog.show();
}

void TransKode::slotConfirmQuit()
{
	if ( m_worker.jobsQueued() || m_worker.processing() )
		if ( KMessageBox::Yes != KMessageBox::questionYesNo(
				0,
				i18n( "There are pending jobs. Really quit?" ),
				i18n( "Confirm action" ) + " - transKode" ) )
			return;

	saveConfig();
	kapp->quit();
}

void TransKode::slotToggleFilterToolBar( bool visible )
{
	KListViewSearchLineWidget* searchLineWidget = (KListViewSearchLineWidget*)m_searchAction->widget();
	if ( !searchLineWidget )
		return;

	KListViewSearchLine* searchLine = searchLineWidget->searchLine();
	if ( !searchLine )
		return;

	// TODO is this right?
	// disable filtering when toolbar is hidden
	searchLine->updateSearch( visible ? QString::null : "" );
}

void TransKode::slotUpdateConfig()
{
	m_config = m_configDialog.getConfig();
	m_config.save( ::locateLocal( "appdata", "transkoderc" ) );

	m_logger.setFile( m_config.generalOpt( GeneralOpt::log ), true );

	bool showTray = m_config.generalOpt( GeneralOpt::show_tray ) == "true";

	if ( showTray )
		m_systemTray->show();
	else
	{
		m_systemTray->hide();
		show();
	}
}

void TransKode::updateState()
{
	int queuedJobs = m_worker.jobsQueued();
	bool transcoding = m_worker.processing();

	if ( !queuedJobs && !transcoding )
	{
		QToolTip::add( m_systemTray, "transKode" );
		setCaption( "" );
	}
	else if ( !m_worker.paused() )
	{
		m_pauseEncodingAction->setText( i18n( "Pause" ) );
		m_pauseEncodingAction->setToolTip( i18n( "Pause transcoding" ) );
		QString caption = queuedJobs > 0 ?
			i18n( "Transcoding (1 job pending)", "Transcoding (%n jobs pending)", queuedJobs ) :
			i18n( "Transcoding" );
		QToolTip::add( m_systemTray, "transKode - " + caption );
		setCaption( caption );
	}
	else
	{
		m_pauseEncodingAction->setText( i18n( "Resume" ) );
		m_pauseEncodingAction->setToolTip( i18n( "Resume transcoding" ) );
		QString caption = queuedJobs > 0 ?
			i18n( "Paused (1 job pending)", "Paused (%n jobs pending)", queuedJobs ) :
			i18n( "Paused" );
		QToolTip::add( m_systemTray, "transKode - " + caption );
		setCaption( caption );
	}
}

/*
void TransKode::setState( TState state )
{
	switch ( state )
	{
		case idle:
			QToolTip::add( m_systemTray, "transKode" );
			setCaption( "" );
			break;
		case transcoding:
			m_pauseEncodingAction->setText( i18n( "Pause" ) );
			m_pauseEncodingAction->setToolTip( i18n( "Pause transcoding" ) );
			QToolTip::add( m_systemTray, "transKode - " + i18n( "Transcoding" ) );
			setCaption( i18n( "Transcoding" ) );
			break;
		case paused:
			m_pauseEncodingAction->setText( i18n( "Resume" ) );
			m_pauseEncodingAction->setToolTip( i18n( "Resume transcoding" ) );
			QToolTip::add( m_systemTray, "transKode - " + i18n( "Paused" ) );
			setCaption( i18n( "Paused" ) );
			break;
	}
}
*/

bool TransKode::queryClose()
{
	if ( m_config.generalOpt( GeneralOpt::show_tray ) == "true" )
	{
		hide();
		return false;
	}
	else
	{
		if ( m_worker.jobsQueued() || m_worker.processing() )
			if ( KMessageBox::Yes != KMessageBox::questionYesNo(
					0,
					i18n( "There are pending jobs. Really quit?" ),
					i18n( "Confirm action - " ) + "transKode" ) )
				return false;

		saveConfig();
		return true;
	}
}

void TransKode::saveConfig()
{
//	kapp->config()->setGroup( "TransKode Settings" );

	m_listView->saveConfig();
}

void TransKode::loadConfig()
{
//	kapp->config()->setGroup( "TransKode Settings" );

	m_listView->loadConfig();
}

#include "transkode.moc"
