/***************************************************************************
 *   Copyright (C) 2006 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "transkodelistview.h"
#include "transkodelistviewitem.h"

#include <qfileinfo.h>
#include <qheader.h>

#include <kurl.h>
#include <kurldrag.h>
#include <kpopupmenu.h>
#include <klocale.h>
#include <kapplication.h>
#include <kfiledialog.h>
#include <kmessagebox.h>


TransKodeListView::TransKodeListView( QWidget *parent, const TransKodeConfig& config, TransKodeFileWorker& worker ):
	KListView( parent, "TransKodeListView" ),
	m_config( config ),
	m_worker( worker ),
	m_lastDir( QDir::homeDirPath() ),
	m_horizMargin( 1 )
{
	addColumn( i18n( "Source Path" ) );
	addColumn( i18n( "Source Directory" ) );
	addColumn( i18n( "Source Filename" ) );
	addColumn( i18n( "Destination Path" ) );
	addColumn( i18n( "Destination Directory" ) );
	addColumn( i18n( "Destination Filename" ) );
	addColumn( i18n( "Profile" ) );
	addColumn( i18n( "Status" ) );
	addColumn( i18n( "Queue Pos" ) );

	for ( int idx = 0; idx < columns(); ++idx )
		setColumnWidthMode( idx, QListView::Manual ),

	setColumnAlignment( c_profile, Qt::AlignCenter );
	setColumnAlignment( c_status_msg, Qt::AlignCenter );
	setColumnAlignment( c_queue, Qt::AlignCenter );


	setShowSortIndicator( true );
	setAllColumnsShowFocus( true );
	setSelectionMode( QListView::Extended );
	setItemsMovable( true );

	setDropVisualizer( true );
	setDropVisualizerWidth( 2 );
	setItemHorizontalMargin( 3 );

	setAcceptDrops( true );
	setDragEnabled( true );
	connect( this, SIGNAL( dropped(QDropEvent*,QListViewItem*) ), this, SLOT( onDropEvent(QDropEvent*,QListViewItem*) ) );

	header()->installEventFilter( this );
	disconnect( header(), SIGNAL( sectionHandleDoubleClicked(int) ), this, 0 );
	connect( header(), SIGNAL( clicked(int) ), this, SLOT( slotSetSortColumn(int) ) );
	connect( header(), SIGNAL( sizeChange(int,int,int) ), this, SLOT( slotRepaint() ) );
	connect( header(), SIGNAL( sectionHandleDoubleClicked(int) ), this, SLOT( slotAdjustColumnWidth(int)) );

	connect( this, SIGNAL( currentChanged(QListViewItem*) ), this, SLOT( slotCurrentChanged(QListViewItem*) ) );
}

TransKodeListViewItem* TransKodeListView::addFile( const QString& file, TransKodeListViewItem* after )
{
	QFileInfo fileInfo( file );
	TransKodeListViewItem* item = new TransKodeListViewItem( this, m_config, file );
	item->moveItem( after );

	connect( item->job(), SIGNAL( mtFinished(long,const QString&,const QString&,int) ), parent(), SLOT( slotJobFinished() ) );

	return item;
}

TransKodeListViewItem* TransKodeListView::addFiles( const QStringList& files, TransKodeListViewItem* after )
{
	for ( QStringList::ConstIterator it = files.begin(); it != files.end(); ++it )
		after = addFile( *it, after );

	return after;
}

void TransKodeListView::selectAll()
{
	//wtf? it won't compile unless i explicitely add base class identifier
	KListView::selectAll( true );
}

void TransKodeListView::refresh()
{
	for ( QListViewItemIterator it( this, QListViewItemIterator::Selected ) ; it.current() != 0; ++it )
	{
		if ( it.current()->rtti() != TRANSKODE )
			continue;

		((TransKodeListViewItem*)it.current())->refresh();
	}
}

void TransKodeListView::clearAll()
{
	QPtrList<TransKodeListViewItem> items;
	for ( QListViewItemIterator it( this ) ; it.current() != 0; ++it )
	{
		if ( it.current()->rtti() != TRANSKODE )
			continue;
		items.append( (TransKodeListViewItem*)it.current() );
	}

	if ( childCount() && KMessageBox::Yes ==
		 KMessageBox::questionYesNo( 0, i18n( "Remove all jobs?" ), i18n( "Confirm action" ) + " - transKode" ) )
	{
		for ( QPtrListIterator<TransKodeListViewItem> it( items ); it.current() != 0; ++it )
			(*it)->remove();

		clear();

		emit allCleared();
		emit finishedCleared();
	}
}

void TransKodeListView::clearFinished()
{
	QPtrList<QListViewItem> itemsToRemove;
	itemsToRemove.setAutoDelete( true );
	for ( QListViewItemIterator it( this ) ; it.current() != 0; ++it )
	{
		if ( it.current()->rtti() != TRANSKODE )
			continue;

		if ( ((TransKodeListViewItem*)it.current())->isDone() )
			itemsToRemove.append( it.current() );
	}

	emit finishedCleared();
	if ( childCount() == 0 )
		emit allCleared();
}

void TransKodeListView::filesInDir( const QString& d, QStringList& files )
{
	static const QStringList extensions = QStringList::split(
		" ",
		"mp3 ogg mpc mpp mp+ aac m4a mp4 flac ogg ape mac spx ra wma wav ac3 mka mp2 ofr la pac bonk tta shn wv" );

	QDir dir( d );
	QFileInfo dirInfo( dir.absPath() );

	QString dirPath = dir.absPath();
	QString parentDirPath = dirInfo.dirPath( true );

	const QFileInfoList* filesInfo = dir.entryInfoList(
		QDir::Files|QDir::Dirs|QDir::Readable|QDir::Hidden|QDir::NoSymLinks );

	for ( QFileInfoListIterator it( *filesInfo ); it.current() != 0; ++it )
	{
		QString curPath = it.current()->absFilePath();
		if ( curPath == dirPath || curPath == parentDirPath )
			continue;

		if ( it.current()->isFile() )
		{
			if ( extensions.contains( it.current()->extension().lower() ) )
				files.append( it.current()->absFilePath() );
		}
		else if ( it.current()->isDir() )
			filesInDir( it.current()->absFilePath(), files );
	}
}

void TransKodeListView::addFiles()
{
	static QString filter(
		"*.mp3 *.MP3 *.ogg *.OGG *.mpc *.MPC *.mpp *.MPP *.mp+ *.MP+ *.aac *.AAC *.m4a *.M4A *.mp4 *.MP4 "
		"*.flac *.FLAC *.ogg *.OGG *.ape *.APE *.mac *.MAC *.wv *.WV *.ofr *.OFR *.tta *.TTA *.shn *.SHN "
		"*.bonk *.BONK *.la *.LA *.spx *.SPX *.wma *.WMA *.wav *.WAV *.ac3 *.AC3 *.mka *.MKA *.mp2 *.MP2 "
		"*.ra *.RA|" + i18n( "All Supported Files" ) +
		"\n*.mp3 *.MP3|" + i18n( "MPEG Layer 3 Audio" ) +
		"\n*.ogg *.OGG|" + i18n( "Ogg Vorbis Audio" ) +
		"\n*.mpc *.MPC *.mpp *.MPP *.mp+ *.MP+|" + i18n( "Musepack Audio" ) +
		"\n*.aac *.AAC *.m4a *.M4A *.mp4 *.MP4|" + i18n( "AAC\\/MP4 Audio" ) +
		"\n*.flac *.FLAC *.ogg *.OGG|" + i18n( "FLAC Audio" ) +
		"\n*.ape *.APE *.mac *.MAC|" + i18n( "Monkey's Audio" ) +
		"\n*.wv *.WV|" + i18n( "WavPack Audio" ) +
		"\n*.pac *.PAC|" + i18n( "LPAC Audio" ) +
		"\n*.ofr *.OFR|" + i18n( "OptimFROG Audio" ) +
		"\n*.bonk *.BONK|" + i18n( "Bonk Audio" ) +
		"\n*.shn *.SHN|" + i18n( "Shorten Audio" ) +
		"\n*.tta *.TTA|" + i18n( "True Audio" ) +
		"\n*.la *.LA|" + i18n( "Lossless Audio" ) +
		"\n*.spx *.SPX|" + i18n( "Speex Audio" ) +
		"\n*.ra *.RA|" + i18n( "Real Audio" ) +
		"\n*.wma *.WMA|" + i18n( "Windows Media Audio" ) +
		"\n*.wav *.WAV|" + i18n( "Windows PCM Audio" ) +
		"\n*.ac3 *.AC3|" + i18n( "Dolby AC3 Audio" ) +
		"\n*.mka *.MKA|" + i18n( "Matroska Audio" ) +
		"\n*.mp2 *.MP2|" + i18n( "MPEG Layer 2 Audio" )
	);

	QStringList files = KFileDialog::getOpenFileNames( m_lastDir, filter, this, i18n( "Add files" ) );
	if ( !files.isEmpty() )
	{
		QFileInfo fileInfo( files[0] );
		m_lastDir = fileInfo.dirPath( true );

		TransKodeListViewItem* item = 0;
		for ( QStringList::Iterator it = files.begin(); it != files.end(); ++it )
		{
			fileInfo.setFile( *it );
			if ( fileInfo.isFile() )
				item = addFile( *it, item );
			else if ( fileInfo.isDir() )
			{
				QStringList nestedFiles;
				filesInDir( *it, nestedFiles );
				for ( QStringList::Iterator nestedIt = nestedFiles.begin(); nestedIt != nestedFiles.end(); ++nestedIt )
					item = addFile( *nestedIt, item );
			}
		}

		emit contentAdded();
	}
}

void TransKodeListView::removeFiles()
{
	QPtrList<TransKodeListViewItem> jobsToRemove;
	for ( QListViewItemIterator it( this, QListViewItemIterator::Selected ) ; it.current() != 0; ++it )
	{
		if ( it.current()->rtti() != TRANSKODE )
			continue;

		jobsToRemove.append( (TransKodeListViewItem*)it.current() );
	}

	if ( jobsToRemove.count() )
	{
		if ( KMessageBox::questionYesNo( 0, i18n( "Remove selected jobs?" ), i18n( "Confirm action" ) + " - transKode" ) != KMessageBox::Yes )
			return;

		for ( QPtrListIterator<TransKodeListViewItem> it( jobsToRemove ); it.current() != 0; ++it )
			(*it)->remove();

		bool itemsFinished = false;
		for ( QListViewItemIterator it( this, QListViewItemIterator::Selected ) ; it.current() != 0; ++it )
		{
			if ( it.current()->rtti() != TRANSKODE )
				continue;

			if ( ((TransKodeListViewItem*)it.current())->isDone() )
			{
				itemsFinished = true;
				break;
			}
		}

		if ( !itemsFinished )
			emit finishedCleared();

		if ( childCount() == 0 )
			emit allCleared();
	}
}


void TransKodeListView::queueFiles()
{
	for ( QListViewItemIterator it( this, QListViewItemIterator::Selected ) ; it.current() != 0; ++it )
	{
		if ( it.current()->rtti() != TRANSKODE )
			continue;

		((TransKodeListViewItem*)it.current())->enqueue( &m_worker );
	}

	emit selectionChanged();
}

void TransKodeListView::dequeueFiles()
{
	bool itemsToCancel = false;
	for ( QListViewItemIterator it( this, QListViewItemIterator::Selected ) ; it.current() != 0; ++it )
	{
		if ( it.current()->rtti() != TRANSKODE )
			continue;

		if ( ((TransKodeListViewItem*)it.current())->isEnqueued() )
		{
			itemsToCancel = true;
			break;
		}
	}

	if ( itemsToCancel )
	{
		if ( KMessageBox::questionYesNo( 0, i18n( "Cancel transcoding of selected files?" ), i18n( "Confirm action" ) + " - transKode" ) != KMessageBox::Yes )
			return;

		for ( QListViewItemIterator it( this, QListViewItemIterator::Selected ) ; it.current() != 0; ++it )
		{
			if ( it.current()->rtti() != TRANSKODE )
				continue;

			((TransKodeListViewItem*)it.current())->cancel();
		}
	}

	emit selectionChanged();
}


// emit a signal with the program status bar text
void TransKodeListView::slotCurrentChanged( QListViewItem* current )
{
	if ( current == 0 || current->rtti() != TRANSKODE )
		emit message( "" );
	else
	{
		TransKodeListViewItem* item = (TransKodeListViewItem*)current;
		if ( ! item->job()->profile().isEmpty() )
		{
			ProfileOptions* opts = m_config.profileOptions( item->job()->profile() );
			if ( opts != 0 )
			{
				emit message(
					item->job()->srcPath() +
					" -> " + item->job()->dstPath() +
					" | " + item->job()->profile() +
					" [" + opts->switches() + "]" );
				delete opts;
				return;
			}
		}
		emit message( item->job()->srcPath() );
	}
}

void TransKodeListView::slotSetSortColumn( int column )
{
	setSortColumn( column );

	m_sortColumns.remove( column );
	m_sortColumns.prepend( column );

	repaint(); // otherwise progress bars get painted anywhere
}

void TransKodeListView::slotAdjustColumnWidth( int column )
{
	adjustColumn( column );
	setColumnWidth( column, columnWidth( column ) + 2*m_horizMargin );
}

void TransKodeListView::slotRepaint()
{
	for ( QListViewItemIterator it( this ); it.current(); ++it )
	{
		if ( it.current()->rtti() != TRANSKODE )
			continue;

		repaintItem( it.current() );
		//((TransKodeListViewItem*)it.current())->repaint();
	}
}

//TODO: needed?
void TransKodeListView::adjustGeometry()
{
	for ( QListViewItemIterator it( this ); it.current(); ++it )
	{
		if ( it.current()->rtti() != TRANSKODE )
			continue;

		((TransKodeListViewItem*)it.current())->adjustGeometry();
	}
}

void TransKodeListView::onDropEvent( QDropEvent* event, QListViewItem* after )
{
	KURL::List urls;
	QString text;
	if ( KURLDrag::decode( event, urls ) && !urls.isEmpty() )
	{
		setSorting( -1 );

		QFileInfo fileInfo;
		TransKodeListViewItem* item = 0;
		for ( KURL::List::ConstIterator it = urls.begin(); it != urls.end(); ++it )
		{
			if ( (*it).protocol() != "file" )
				continue;

			fileInfo.setFile( (*it).path() );
			if ( fileInfo.isFile() )
				item = addFile( (*it).path(), item );
			else if ( fileInfo.isDir() )
			{
				QStringList nestedFiles;
				filesInDir( (*it).path(), nestedFiles );
				for ( QStringList::Iterator nestedIt = nestedFiles.begin(); nestedIt != nestedFiles.end(); ++nestedIt )
					item = addFile( *nestedIt, item );
			}
		}

		emit contentAdded();
	}
	else if ( QTextDrag::decode( event, text ) ) // dragging withing the list
	{
		setSorting( -1 );
		m_sortColumns.clear();

		QListViewItem* current = currentItem();

		QListViewItem* item;
		QStringList list = QStringList::split( "\n", text );
		// How dangerous is this kind of manipulation?... probably pretty nasty
		for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
		{
			item = static_cast<QListViewItem*>((void *)(*it).toLong( 0 ));
			//item->moveItem( after ); // this one doesn't work when after is 0
			moveItem( item, 0, after );
			after = item;
		}

		setCurrentItem( current );
	}
}

bool TransKodeListView::acceptDrag( QDropEvent* event ) const
{
	return KURLDrag::canDecode( event ) || ( QTextDrag::canDecode( event ) && event->source() == this );
}

QDragObject* TransKodeListView::dragObject()
{
	QString text;

	for ( QListViewItemIterator it( this ); it.current(); ++it )
		if ( it.current()->isSelected() )
			text += QString::number( (long int)it.current() ) + "\n";

	return new QTextDrag( text, this );
}

// Method (mostly) taken from amaroK
bool TransKodeListView::eventFilter( QObject* object, QEvent* event )
{
	if ( object == header() && event->type() == QEvent::MouseButtonPress )
	{
		QMouseEvent* me = static_cast<QMouseEvent*>(event);
		if ( me->button() == Qt::RightButton )
		{
			enum { HIDE = 1000, CUSTOM };
	
			const int columnUnderMouse = header()->sectionAt( me->pos().x() );
	
			KPopupMenu popup;
			popup.setCheckable( true );
			popup.insertItem( i18n( "Hide This Column" ), HIDE ); //TODO
			popup.setItemEnabled( HIDE, columnUnderMouse != -1 );
	
			for ( int i = 0; i < columns(); ++i )
			{
				popup.insertItem( columnText( i ), i, i + 1 );
				popup.setItemChecked( i, columnWidth( i ) != 0 );
			}
	
			//do last so it doesn't get the first id
			popup.insertTitle( "transKode " + i18n( "Columns" ), /*id*/ -1, /*index*/ 1 );
	
			int col = popup.exec( me->globalPos() );
	
			if ( col == HIDE )
			{
				hideColumn( columnUnderMouse );
				QResizeEvent e( size(), QSize() );
				viewportResizeEvent( &e );
			}
			else if( col != -1 )
			{
				//TODO can result in massively wide column appearing!
				if ( columnWidth( col ) == 0 )
				{
					slotAdjustColumnWidth( col );
					header()->setResizeEnabled( true, col );
				}
				else
				{
					hideColumn( col );
					header()->setResizeEnabled( false, col );
				}
			}
	
			return true; //eat event
		}
	}

	return KListView::eventFilter( object, event ); //allow the header to process this
}


void TransKodeListView::contextMenuEvent( QContextMenuEvent* e )
{
	QPtrList<TransKodeListViewItem> items;
	for ( QListViewItemIterator it( this, QListViewItemIterator::Selected ) ; it.current() != 0; ++it )
	{
		if ( it.current()->rtti() != TRANSKODE )
			continue;

		if ( !((TransKodeListViewItem*)it.current())->isEnqueued() )
			items.append( ((TransKodeListViewItem*)it.current()) );
	}

	if ( items.count() <= 0 )
		return;

	KPopupMenu popup;

	QStringList profiles = m_config.validProfiles();
	QStringList showProfiles;
	popup.insertTitle( i18n( "Set Profile" ), -1 );
	if ( profiles.count() > 0 )
	{
		int id = 0;
		for ( QStringList::Iterator it = profiles.begin(); it != profiles.end(); ++it )
		{
			ProfileOptions* options = m_config.profileOptions( *it );
			if ( options != 0 )
			{
				if ( options->show() )
				{
					popup.insertItem( *it, id++ );
					showProfiles.append( *it );
				}
				delete options;
			}
		}
	}
	else
	{
		popup.insertItem( i18n( "No valid profiles available" ), 0 );
		popup.setItemEnabled( 0, false );
	}

	int id = popup.exec( e->globalPos() );
	if ( id >= 0 )
	{
		for ( QPtrList<TransKodeListViewItem>::Iterator it = items.begin(); it != items.end(); ++it )
			(*it)->setProfile( showProfiles[id] );

		emit selectionChanged();
	}
}

void TransKodeListView::setItemHorizontalMargin( uint margin )
{
	m_horizMargin = margin;
}

uint TransKodeListView::itemHorizontalMargin()
{
	return m_horizMargin;
}

void TransKodeListView::loadConfig()
{
	QString prevGroup = kapp->config()->group();
	kapp->config()->setGroup( "TransKodeListView Settings" );

	restoreLayout( kapp->config(), "TransKodeListView Settings" );
	m_sortColumns = kapp->config()->readIntListEntry( "SortColumns" );

	m_lastDir = kapp->config()->readPathEntry( "LastDir", QDir::homeDirPath() );

	kapp->config()->setGroup( prevGroup );
}

void TransKodeListView::saveConfig()
{
	QString prevGroup = kapp->config()->group();
	kapp->config()->setGroup( "TransKodeListView Settings" );

	saveLayout( kapp->config(), "TransKodeListView Settings" );
	kapp->config()->writeEntry( "SortColumns", m_sortColumns );

	kapp->config()->writePathEntry( "LastDir", m_lastDir );

	kapp->config()->setGroup( prevGroup );
}


#include "transkodelistview.moc"
