/***************************************************************************
 *   Copyright (C) 2005 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "transkodeconfig.h"
#include <transkodepluginsmanager.h>

#include <qfileinfo.h>
#include <qdir.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qregexp.h>

#include <ksimpleconfig.h>

TransKodeConfig::TransKodeConfig( const QString& defaultFile ):
	QObject(),
	m_configMutex( true ), // recursive mutex
	m_inited( false )
{
	m_defaultFile = String::deepCopy( defaultFile );

	reloadEncodeTypes();

	loadDefaults();

	m_inited = true;
}

TransKodeConfig::TransKodeConfig( const TransKodeConfig& cfg ):
	QObject(),
	m_configMutex( true ), // recursive mutex
	m_inited( true )
{
	cfg.m_configMutex.lock();

	m_defaultFile = String::deepCopy( cfg.m_defaultFile );

	for ( int idx = 0; idx < GeneralOpt::SIZE; ++idx )
		m_generalOptions[idx] = String::deepCopy( cfg.m_generalOptions[idx] );

	for ( int idx = 0; idx < Tag::SIZE; ++idx )
		m_tagDefaults[idx] = String::deepCopy( cfg.m_tagDefaults[idx] );

	for ( int idx = 0; idx < Tag::SIZE; ++idx )
		m_tagCases[idx] = cfg.m_tagCases[idx];

	for ( QMap<QString,bool>::ConstIterator it = cfg.m_suppEncodeTypes.begin(); it != cfg.m_suppEncodeTypes.end(); ++it )
		m_suppEncodeTypes[String::deepCopy(it.key())] = it.data();

	for ( QMap<QString,ProfileOptions>::ConstIterator it = cfg.m_profiles.begin(); it != cfg.m_profiles.end(); ++it )
	{
		m_profiles[String::deepCopy( it.key() )] = it.data();
		m_profiles[it.key()].setDefaultNamingScheme( cfg.m_generalOptions[GeneralOpt::naming_scheme] );
	}

	cfg.m_configMutex.unlock();
}

TransKodeConfig& TransKodeConfig::operator=( const TransKodeConfig& c )
{
	if ( this == &c )
		return *this;

	// This seems inefficient, but is the best way (I could think of) not to risk deadlock
	TransKodeConfig cfg( c );

	m_configMutex.lock();

	bool emitChanged = false;
	bool emitChangedDefault = false;
	QString prevDefaultFile;
	QValueList<int> changedOptions;
	QValueList<int> changedDefaults;
	QValueList<int> changedCases;
	QStringList removedSuppEncodeType;
	QStringList addedSuppEncodeType;
	QStringList removedProf;
	QStringList addedProf;
	QStringList changedProf;

	setDefaultFile( cfg.m_defaultFile, emitChanged, prevDefaultFile );

	for ( QMap<QString,ProfileOptions>::ConstIterator it = cfg.m_profiles.begin(); it != cfg.m_profiles.end(); ++it )
	{
		if ( ! m_profiles.contains( it.key() ) ) // profile was added
		{
			m_profiles[it.key()] = it.data();
			addedProf.append( it.key() );
			emitChanged = true;
		}
		else if ( m_profiles[it.key()] != it.data() ) // profile was changed
		{
			m_profiles[it.key()] = it.data();
			changedProf.append( it.key() );
			emitChanged = true;
		}
		m_profiles[it.key()].setDefaultNamingScheme( m_generalOptions[GeneralOpt::naming_scheme] );
		m_profiles[it.key()].setNamingScheme( it.data().displayNamingScheme() );
	}

	for ( QMap<QString,ProfileOptions>::Iterator it = m_profiles.begin(); it != m_profiles.end(); ++it )
		if ( ! cfg.m_profiles.contains( it.key() ) ) // profile was removed
			removedProf.append( it.key() );
	for ( QStringList::Iterator it = removedProf.begin(); it != removedProf.end(); ++it )
		m_profiles.remove( *it );
	if ( removedProf.size() )
		emitChanged = true;


	for ( int idx = 0; idx < GeneralOpt::SIZE; ++idx )
		setGeneralOpt( (GeneralOpt::TGeneralOpt)idx, cfg.m_generalOptions[idx], emitChanged, changedOptions, changedProf );

	for ( QMap<QString,bool>::ConstIterator it = cfg.m_suppEncodeTypes.begin(); it != cfg.m_suppEncodeTypes.end(); ++it )
	{
		if ( m_suppEncodeTypes[it.key()] != it.data() )
		{
			if ( it.data() )
				addedSuppEncodeType.append( it.key() );
			else
				removedSuppEncodeType.append( it.key() );
			m_suppEncodeTypes[it.key()] = it.data();
		}
	}


	for ( int idx = 0; idx < Tag::SIZE; ++idx )
		setTagDefault( (Tag::TTag)idx, cfg.m_tagDefaults[idx], emitChanged, changedDefaults );

	for ( int idx = 0; idx < Tag::SIZE; ++idx )
		setTagCase( (Tag::TTag)idx, cfg.m_tagCases[idx], emitChanged, changedCases );

	m_configMutex.unlock();

	if ( emitChanged )						emit changed();
	if ( emitChangedDefault )				emit changedDefaultFile( prevDefaultFile );
	if ( changedOptions.size() )			emit changedGeneralOpts( changedOptions );
	if ( changedDefaults.size() )			emit changedTagDefaults( changedDefaults );
	if ( changedCases.size() )				emit changedTagCases( changedCases );
	if ( addedSuppEncodeType.size() )		emit addedSuppEncodeTypes( addedSuppEncodeType );
	if ( removedSuppEncodeType.size() )		emit removedSuppEncodeTypes( removedSuppEncodeType );
	if ( addedProf.size() )					emit addedProfiles( addedProf );
	if ( removedProf.size() )				emit removedProfiles( removedProf );
	if ( changedProf.size() )				emit changedProfiles( changedProf );

	return *this;
}


QString TransKodeConfig::generalOpt( GeneralOpt::TGeneralOpt option ) const
{
	if ( option >= GeneralOpt::SIZE )
		return QString( "" );

	m_configMutex.lock();

	QString ret = String::deepCopy( m_generalOptions[option] );

	m_configMutex.unlock();

	return ret;
}

QString TransKodeConfig::programPath( const QString& program, bool quoted ) const
{
	QString path = TransKodePluginsManager::programPath( program );

	return quoted ? String::quoteForShell( path ) : path;
}

CaseFunc::TCaseFunc TransKodeConfig::tagCase( Tag::TTag tag ) const
{
	if ( tag >= Tag::SIZE )
		return CaseFunc::leaveasis;

	m_configMutex.lock();

	CaseFunc::TCaseFunc ret = m_tagCases[tag];

	m_configMutex.unlock();

	return ret;
}

QString TransKodeConfig::tagDefault( Tag::TTag tag ) const
{
	if ( tag >= Tag::SIZE )
		return QString();

	m_configMutex.lock();

	QString ret = String::deepCopy( m_tagDefaults[tag] );

	m_configMutex.unlock();

	return ret;
}

QStringList TransKodeConfig::profiles() const
{
	m_configMutex.lock();

	QStringList ret;
	for ( QMap<QString,ProfileOptions>::ConstIterator it = m_profiles.begin(); it != m_profiles.end(); ++it )
		ret.append( String::deepCopy( it.key() ) );

	m_configMutex.unlock();

	return ret;
}

bool TransKodeConfig::encodeTypeIsSupported( const QString& encodeType ) const
{
	if ( encodeType == "Bypass" )
		return true;

	m_configMutex.lock();

	bool ret = m_suppEncodeTypes.contains( encodeType ) ? m_suppEncodeTypes[encodeType] : false;

	m_configMutex.unlock();

	return ret;
}

QString TransKodeConfig::supportedEncodingExtensions() const
{
	m_configMutex.lock();

	QStringList extensions;

	for ( QMap<QString,bool>::ConstIterator it = m_suppEncodeTypes.begin(); it != m_suppEncodeTypes.end(); ++it )
	{
		if ( !it.data() )
			continue;

		const TransKodeEncoderPlugin* plugin = TransKodePluginsManager::encoderPlugin( it.key() );
		if ( plugin == 0 )
			continue;

		QStringList pluginExtensions = plugin->extensions();
		for ( QStringList::Iterator it = pluginExtensions.begin(); it != pluginExtensions.end(); ++it )
			if ( !extensions.contains( *it) )
				extensions.append( *it );
	}

	m_configMutex.unlock();

	return extensions.join( " " );
}

/*bool TransKodeConfig::programIsSupported( Program::TProgram program ) const
{
	m_configMutex.lock();

	bool ret = KStandardDirs::findExe( m_programPaths[program] ) != QString::null;

	m_configMutex.unlock();

	return ret;
}*/

/*bool TransKodeConfig::baseDirIsValid() const
{
	m_configMutex.lock();

	bool ret;

	QDir absBaseDir( m_generalOptions[GeneralOpt::base_dir] );
	absBaseDir.convertToAbs();
	QFileInfo dirInfo( absBaseDir.path() );
	ret = dirInfo.isDir() && dirInfo.isReadable() && dirInfo.isWritable() && dirInfo.isExecutable();

	m_configMutex.unlock();

	return ret;
}*/

/*bool TransKodeConfig::profileHasValidBaseDir( const QString& profile ) const
{
	m_configMutex.lock();

	bool ret;

	if ( m_profiles.contains( profile ) )
		ret = m_profiles[profile].haveValidBaseDir();
	else
		ret = false;

	m_configMutex.unlock();

	return ret;
}

bool TransKodeConfig::profileHasValidNamingScheme( const QString& profile ) const
{
	m_configMutex.lock();

	bool ret;

	if ( m_profiles.contains( profile ) )
		ret = m_profiles[profile].haveValidNamingScheme();
	else
		ret = false;

	m_configMutex.unlock();

	return ret;
}*/

bool TransKodeConfig::profileIsValid( const QString& profile ) const
{
	ProfileOptions* options = profileOptions( profile );
	if ( options == 0 )
		return false;
	else
	{
		bool valid = profileOptionsAreValid( options );
		delete options;
		return valid;
	}
}

bool TransKodeConfig::profileOptionsAreValid( const ProfileOptions* options ) const
{
	if ( options == 0 )
		return false;

	return options->haveValidNamingScheme() && encodeTypeIsSupported( options->encodeType() );
}

QStringList TransKodeConfig::validProfiles() const
{
	m_configMutex.lock();

	QStringList ret;
	for ( QMap<QString,ProfileOptions>::ConstIterator it = m_profiles.begin(); it != m_profiles.end(); ++it )
		if ( profileOptionsAreValid( &it.data() ) )
			ret.append( String::deepCopy( it.key() ) );

	m_configMutex.unlock();

	return ret;
}

QStringList TransKodeConfig::invalidProfiles() const
{
	m_configMutex.lock();

	QStringList ret;
	for ( QMap<QString,ProfileOptions>::ConstIterator it = m_profiles.begin(); it != m_profiles.end(); ++it )
		if ( ! profileOptionsAreValid( &it.data() ) )
			ret.append( String::deepCopy( it.key() ) );

	m_configMutex.unlock();

	return ret;
}


ProfileOptions* TransKodeConfig::profileOptions( const QString& profile ) const
{
	m_configMutex.lock();

	ProfileOptions* ret;
	if ( m_profiles.contains( profile ) )
		ret = new ProfileOptions( m_profiles[profile] );
	else
		ret = 0;

	m_configMutex.unlock();

	return ret;
}

QString TransKodeConfig::defaultFile() const
{
	m_configMutex.lock();

	QString ret = String::deepCopy( m_defaultFile );

	m_configMutex.unlock();

	return ret;
}


void TransKodeConfig::setGeneralOpt(
	GeneralOpt::TGeneralOpt option,
	const QString& value,
	bool& emitChanged,
	QValueList<int>& changedOptions,
	QStringList& changedProfiles )
{
	if ( option >= GeneralOpt::SIZE )
		return;

	QString aux = value;

	if ( option == GeneralOpt::proc_priority )
	{
		int priority = aux.toInt();
		if ( priority < 0 )
			aux = "0";
		else if ( priority > 19 )
			aux = "19";
	}
	else if ( option == GeneralOpt::naming_scheme || option == GeneralOpt::log )
		aux.replace( QRegExp( "^(~|\\$HOME)" ), QDir::homeDirPath() );
	else if ( option == GeneralOpt::temp_dir && value.isEmpty() )
		aux = System::tempDir();

	if ( m_generalOptions[option] != aux )
	{
		m_generalOptions[option] = String::deepCopy( aux );
		changedOptions.append( option );
		emitChanged = true;

		if ( option == GeneralOpt::naming_scheme )
		{
			for ( QMap<QString,ProfileOptions>::Iterator it = m_profiles.begin(); it != m_profiles.end(); ++it )
			{
				if ( it.data().usesDefaultNamingScheme() )
					changedProfiles.append( it.key() );
				it.data().setDefaultNamingScheme( m_generalOptions[GeneralOpt::naming_scheme] );
			}
		}
	}
}

void TransKodeConfig::setGeneralOpt( GeneralOpt::TGeneralOpt option, const QString& value )
{
	m_configMutex.lock();

	bool emitChanged = false;
	QValueList<int> changedOptions;
	QStringList changedProf;
	setGeneralOpt( option, value, emitChanged, changedOptions, changedProf );

	m_configMutex.unlock();

	if ( emitChanged )				emit changed();
	if ( changedProf.size() )		emit changedProfiles( changedProf );
	if ( changedOptions.size() )	emit changedGeneralOpts( changedOptions );
}

void TransKodeConfig::reloadEncodeTypes()
{
	// TODO need to improve this to allow for dynamic loading of plugins

	QStringList pluginNames = TransKodePluginsManager::encoderPluginNames();
	m_suppEncodeTypes.clear();
	for ( QStringList::ConstIterator it = pluginNames.begin(); it != pluginNames.end(); ++it )
	{
		const TransKodeEncoderPlugin* plugin = TransKodePluginsManager::encoderPlugin( *it );
		if ( plugin != 0 )
			m_suppEncodeTypes[*it] = plugin->isLocallySupported();
	}
}

void TransKodeConfig::setProgramPath(
	const QString& program,
	const QString& path,
	bool& emitChanged,
	QStringList& addedSuppEncTypes,
	QStringList& removedSuppEncTypes )
{
	TransKodePluginsManager::setProgramPath( program, path );

	for ( QMap<QString,bool>::Iterator it = m_suppEncodeTypes.begin(); it != m_suppEncodeTypes.end(); ++it )
	{
		const TransKodeEncoderPlugin* plugin = TransKodePluginsManager::encoderPlugin( it.key() );
		if ( plugin == 0 )
		{
			if ( it.data() )
			{
				emitChanged = true;
				m_suppEncodeTypes[it.key()] = false;
				removedSuppEncTypes.append( it.key() );
			}
		}
		else if ( plugin->isLocallySupported() != it.data() )
		{
			emitChanged = true;
			if ( it.data() )
			{
				m_suppEncodeTypes[it.key()] = false;
				removedSuppEncTypes.append( it.key() );
			}
			else
			{
				m_suppEncodeTypes[it.key()] = true;
				addedSuppEncTypes.append( it.key() );
			}
		}
	}
}


void TransKodeConfig::setProgramPath( const QString& program, const QString& value )
{
	m_configMutex.lock();

	bool emitChanged = false;
	QStringList addedSuppEncTypes;
	QStringList removedSuppEncTypes;
	setProgramPath( program, value, emitChanged, addedSuppEncTypes, removedSuppEncTypes );

	m_configMutex.unlock();

	if ( emitChanged )					emit changed();
	if ( removedSuppEncTypes.size() )	emit removedSuppEncodeTypes( removedSuppEncTypes );
	if ( addedSuppEncTypes.size() )		emit addedSuppEncodeTypes( addedSuppEncTypes );
}

void TransKodeConfig::setTagCase( Tag::TTag tag, CaseFunc::TCaseFunc value, bool& emitChanged, QValueList<int>& changedCases )
{
	if ( tag >= Tag::SIZE )
		return;

	if ( m_tagCases[tag] != value )
	{
		m_tagCases[tag] = value;
		changedCases.append( tag );
		emitChanged = true;
	}
}

void TransKodeConfig::setTagCase( Tag::TTag tag, CaseFunc::TCaseFunc value )
{
	m_configMutex.lock();

	bool emitChanged = false;
	QValueList<int> changedCases;
	setTagCase( tag, value, emitChanged, changedCases );

	m_configMutex.unlock();

	if ( emitChanged )				emit changed();
	if ( changedCases.size() )		emit changedTagCases( changedCases );
}

void TransKodeConfig::setTagDefault( Tag::TTag tag, const QString& value, bool& emitChanged, QValueList<int>& changedDefaults )
{
	if ( tag >= Tag::SIZE )
		return;

	if ( m_tagDefaults[tag] != value )
	{
		m_tagDefaults[tag] = String::deepCopy( value );
		changedDefaults.append( tag );
		emitChanged = true;
	}
}

void TransKodeConfig::setTagDefault( Tag::TTag tag, const QString& value )
{
	m_configMutex.lock();

	bool emitChanged = false;
	QValueList<int> changedDefaults;
	setTagDefault( tag, value, emitChanged, changedDefaults );

	m_configMutex.unlock();

	if ( emitChanged )				emit changed();
	if ( changedDefaults.size() )	emit changedTagDefaults( changedDefaults );
}

void TransKodeConfig::setProfileOptions(
	const QString& profile,
	const ProfileOptions& opts,
	bool& emitChanged,
	QStringList& addedProfiles,
	QStringList& changedProfiles )
{
	if ( ! m_profiles.contains( profile ) )
	{
		m_profiles[String::deepCopy( profile )] = opts;
		addedProfiles.append( String::deepCopy( profile ) );
		emitChanged = true;
	}
	else if ( m_profiles[ profile ] != opts )
	{
		m_profiles[profile] = opts;
		changedProfiles.append( String::deepCopy( profile ) );
		emitChanged = true;
	}

	m_profiles[profile].setDefaultNamingScheme( m_generalOptions[GeneralOpt::naming_scheme] );
	m_profiles[profile].setNamingScheme( opts.displayNamingScheme() );
}


void TransKodeConfig::setProfileOptions( const QString& profile, const ProfileOptions& opts )
{
	m_configMutex.lock();

	bool emitChanged = false;
	QStringList addedProf;
	QStringList changedProf;
	setProfileOptions( profile, opts, emitChanged, addedProf, changedProf );

	m_configMutex.unlock();

	if ( emitChanged )				emit changed();
	if ( addedProf.size() )			emit addedProfiles( addedProf );
	if ( changedProf.size() )		emit changedProfiles( changedProf );
}

void TransKodeConfig::removeProfile( const QString& profile, bool& emitChanged, QStringList& removedProfiles )
{
	if ( m_profiles.contains( profile ) )
	{
		m_profiles.remove( profile );
		removedProfiles.append( String::deepCopy( profile ) );
		emitChanged = true;
	}
}

void TransKodeConfig::removeProfile( const QString& profile )
{
	m_configMutex.lock();

	bool emitChanged = false;
	QStringList removedProf;
	removeProfile( profile, emitChanged, removedProf );

	m_configMutex.unlock();

	if ( emitChanged )			emit changed();
	if ( removedProf.size() )	emit removedProfiles( String::deepCopy( removedProf ) );
}

void TransKodeConfig::clearProfiles()
{
	m_configMutex.lock();

	QStringList removedProf;
	for ( QMap<QString,ProfileOptions>::Iterator it = m_profiles.begin(); it != m_profiles.end(); ++it )
		removedProf.append( it.key() );
	m_profiles.clear();

	m_configMutex.unlock();

	if ( removedProf.size() )
	{
		emit changed();
		emit removedProfiles( String::deepCopy( removedProf ) );
	}
}

void TransKodeConfig::setDefaultFile( const QString& defaultFile, bool& emitChangedDefault, QString& prevDefaultFile )
{
	if ( m_defaultFile != defaultFile )
	{
		emitChangedDefault = true;
		prevDefaultFile = m_defaultFile;
		m_defaultFile = String::deepCopy( defaultFile );
	}
}

void TransKodeConfig::setDefaultFile( const QString& defaultFile )
{
	m_configMutex.lock();

	bool emitChangedDefault = false;
	QString prevDefaultFile;
	setDefaultFile( defaultFile, emitChangedDefault, prevDefaultFile );

	m_configMutex.unlock();

	if ( emitChangedDefault )	emit changedDefaultFile( prevDefaultFile );
}

void TransKodeConfig::loadDefaults()
{
	if ( load( defaultFile() ) )
		return;

	m_configMutex.lock();

	QStringList removedProf, changedProf;
	for ( QMap<QString,ProfileOptions>::Iterator it = m_profiles.begin(); it != m_profiles.end(); ++it )
		removedProf.append( String::deepCopy( it.key() ) );
	m_profiles.clear();

	bool emitChanged = false;
	QValueList<int> changedOptions;
	QValueList<int> changedDefaults;
	QValueList<int> changedCases;
	QStringList addedEncTypes;
	QStringList removedEncTypes;

	setGeneralOpt(
		GeneralOpt::naming_scheme,
		QDir::homeDirPath() + "/" + CaseFunc::toQString( CaseFunc::titlecase ) +
			"{%{artist}/%{year} - %{album}/%{track##} - %{title}}." +
			CaseFunc::toQString( CaseFunc::lowercase ) + "{%{dst_ext}}",
		emitChanged,
		changedOptions,
		changedProf );
	setGeneralOpt( GeneralOpt::notify, "true", emitChanged, changedOptions, changedProf );
	setGeneralOpt( GeneralOpt::confirm, "true", emitChanged, changedOptions, changedProf );
	setGeneralOpt( GeneralOpt::overwrite, "false", emitChanged, changedOptions, changedProf );
	setGeneralOpt( GeneralOpt::case_insensitive, "true", emitChanged, changedOptions, changedProf );
	setGeneralOpt( GeneralOpt::replace_chars, "*?\\:|<>", emitChanged, changedOptions, changedProf );
	setGeneralOpt( GeneralOpt::rip_duration, "600", emitChanged, changedOptions, changedProf );
	setGeneralOpt( GeneralOpt::rip_incomplete, "true", emitChanged, changedOptions, changedProf );
	setGeneralOpt( GeneralOpt::proc_priority, "0", emitChanged, changedOptions, changedProf );
	setGeneralOpt( GeneralOpt::show_tray, "true", emitChanged, changedOptions, changedProf );
	setGeneralOpt( GeneralOpt::log, "", emitChanged, changedOptions, changedProf );
	setGeneralOpt( GeneralOpt::temp_dir, System::tempDir(), emitChanged, changedOptions, changedProf );

	setTagDefault( Tag::artist, "unknown_artist", emitChanged, changedDefaults );
	setTagDefault( Tag::album, "unknown_album", emitChanged, changedDefaults );
	setTagDefault( Tag::year, "XXXX", emitChanged, changedDefaults );
	setTagDefault( Tag::track, "XX", emitChanged, changedDefaults );
	setTagDefault( Tag::title, "unknown_title", emitChanged, changedDefaults );
	setTagDefault( Tag::genre, "unknown_genre", emitChanged, changedDefaults );
	setTagDefault( Tag::comment, "no_comment", emitChanged, changedDefaults );

	setTagCase( Tag::artist, CaseFunc::titlecase, emitChanged, changedDefaults );
	setTagCase( Tag::album, CaseFunc::titlecase, emitChanged, changedDefaults );
	setTagCase( Tag::year, CaseFunc::leaveasis, emitChanged, changedDefaults );
	setTagCase( Tag::track, CaseFunc::leaveasis, emitChanged, changedDefaults );
	setTagCase( Tag::title, CaseFunc::titlecase, emitChanged, changedDefaults );
	setTagCase( Tag::genre, CaseFunc::titlecase, emitChanged, changedDefaults );
	setTagCase( Tag::comment, CaseFunc::leaveasis, emitChanged, changedDefaults );

	QStringList programs = TransKodePluginsManager::programs();
	for ( QStringList::ConstIterator it = programs.begin(); it != programs.end(); ++it )
		setProgramPath( *it, TransKodePluginsManager::programPath( *it ), emitChanged, addedEncTypes, removedEncTypes );

	m_configMutex.unlock();

	if ( m_inited )
	{
		if ( emitChanged )					emit changed();
		if ( changedOptions.size() )		emit changedGeneralOpts( changedOptions );
		if ( changedDefaults.size() )		emit changedTagDefaults( changedDefaults );
		if ( changedCases.size() )			emit changedTagCases( changedCases );
		if ( addedEncTypes.size() )			emit addedSuppEncodeTypes( addedEncTypes );
		if ( removedEncTypes.size() )		emit removedSuppEncodeTypes( removedEncTypes );
		if ( removedProf.size() )			emit removedProfiles( removedProf );
	}
}

bool TransKodeConfig::load( const QString& path )
{
	if ( !System::isReadable( path ) )
		return false;

	m_configMutex.lock();

	QDir absPath( path );
	absPath.convertToAbs();
	KSimpleConfig* config = new KSimpleConfig( absPath.path(), true );

	bool emitChanged = false;
	QValueList<int> changedOptions;
	QValueList<int> changedDefaults;
	QValueList<int> changedCases;
	QStringList addedEncTypes;
	QStringList removedEncTypes;
	QStringList removedProf;
	QStringList addedProf;
	QStringList changedProf;

	QStringList profilesGroups = config->groupList();
	QStringList settingsGroups;
	if ( profilesGroups.contains( "General" ) )
		{ profilesGroups.remove( "General" ); settingsGroups.append( "General" ); }
	if ( profilesGroups.contains( "Program Locations" ) )
		{ profilesGroups.remove( "Program Locations" ); settingsGroups.append( "Program Locations" ); }
	if ( profilesGroups.contains( "Tag Defaults" ) )
		{ profilesGroups.remove( "Tag Defaults" ); settingsGroups.append( "Tag Defaults" ); }
	if ( profilesGroups.contains( "Tag Cases" ) )
		{ profilesGroups.remove( "Tag Cases" ); settingsGroups.append( "Tag Cases" ); }

	profilesGroups.sort();
	for ( QStringList::Iterator it = profilesGroups.begin(); it != profilesGroups.end(); ++it )
	{
		config->setGroup( *it );

		ProfileOptions profileOpts;
		profileOpts.readFromConfig( config );

		if ( m_profiles.contains( *it ) )
		{
			if ( m_profiles[*it] != profileOpts ) // profile was changed
			{
				m_profiles[*it] = profileOpts;
				changedProf.append( *it );
				emitChanged = true;
			}
		}
		else // profile is added
		{
			m_profiles[*it] = profileOpts;
			addedProf.append( *it );
			emitChanged = true;
		}
	}

	for ( QMap<QString,ProfileOptions>::Iterator it = m_profiles.begin(); it != m_profiles.end(); ++it )
		if ( ! profilesGroups.contains( it.key() ) ) // profile was removed
			removedProf.append( it.key() );
	for ( QStringList::Iterator it = removedProf.begin(); it != removedProf.end(); ++it )
		m_profiles.remove( *it );
	if ( removedProf.size() )
		emitChanged = true;

	if ( settingsGroups.contains( "General" ) )
	{
		config->setGroup( "General" );

		for ( int idx = 0; idx < GeneralOpt::SIZE; ++idx )
			setGeneralOpt(
				(GeneralOpt::TGeneralOpt)idx,
				config->readEntry( GeneralOpt::toQString( (GeneralOpt::TGeneralOpt)idx ), m_generalOptions[idx] ),
				emitChanged,
				changedOptions,
				changedProf );
	}

	if ( settingsGroups.contains( "Program Locations" ) )
	{
		config->setGroup( "Program Locations" );

		QStringList programs = TransKodePluginsManager::programs();
		for ( QStringList::ConstIterator it = programs.begin(); it != programs.end(); ++it )
			setProgramPath( *it, config->readEntry( *it, TransKodePluginsManager::programPath( *it ) ), emitChanged, addedEncTypes, removedEncTypes );
	}

	if ( settingsGroups.contains( "Tag Defaults" ) )
	{
		config->setGroup( "Tag Defaults" );

		for ( int idx = 0; idx < Tag::SIZE; ++idx )
			setTagDefault( (Tag::TTag)idx,
			config->readEntry( Tag::toQString( (Tag::TTag)idx ), m_tagDefaults[idx] ),
			emitChanged,
			changedDefaults );
	}

	if ( settingsGroups.contains( "Tag Cases" ) )
	{
		config->setGroup( "Tag Cases" );

		for ( int idx = 0; idx < Tag::SIZE; ++idx )
			setTagCase( (Tag::TTag)idx,
			CaseFunc::fromQString( config->readEntry( Tag::toQString( (Tag::TTag)idx ), CaseFunc::toQString( m_tagCases[idx] ) ) ),
			emitChanged,
			changedCases );
	}

	m_configMutex.unlock();

	delete config;


	if ( m_inited )
	{
		if ( emitChanged )					emit changed();
		if ( changedOptions.size() )		emit changedGeneralOpts( changedOptions );
		if ( changedDefaults.size() )		emit changedTagDefaults( changedDefaults );
		if ( changedCases.size() )			emit changedTagCases( changedCases );
		if ( addedEncTypes.size() )			emit addedSuppEncodeTypes( addedEncTypes );
		if ( removedEncTypes.size() )		emit removedSuppEncodeTypes( removedEncTypes );
		if ( addedProf.size() )				emit addedProfiles( addedProf );
		if ( removedProf.size() )			emit removedProfiles( removedProf );
		if ( changedProf.size() )			emit changedProfiles( changedProf );
	}

	return true;
}

// Saves configuration to file
bool TransKodeConfig::save( const QString& path ) const
{
	QDir absPath( path );
	absPath.convertToAbs();
	System::remove( absPath.path() );
	KSimpleConfig* config = new KSimpleConfig( absPath.path() );

	m_configMutex.lock();

	config->setGroup( "General" );
	for ( int idx = 0; idx < GeneralOpt::SIZE; ++idx )
		config->writeEntry( GeneralOpt::toQString( (GeneralOpt::TGeneralOpt)idx ), m_generalOptions[idx] );

	config->setGroup( "Program Locations" );
	QStringList programs = TransKodePluginsManager::programs();
	for ( QStringList::ConstIterator it = programs.begin(); it != programs.end(); ++it )
		config->writeEntry( *it, TransKodePluginsManager::programPath( *it ) );

	config->setGroup( "Tag Defaults" );
	for ( int idx = 0; idx < Tag::SIZE; ++idx )
		config->writeEntry( Tag::toQString( (Tag::TTag)idx ), m_tagDefaults[idx] );

	config->setGroup( "Tag Cases" );
	for ( int idx = 0; idx < Tag::SIZE; ++idx )
		config->writeEntry( Tag::toQString( (Tag::TTag)idx ), CaseFunc::toQString( (CaseFunc::TCaseFunc)m_tagCases[idx]) );

	for ( QMap<QString,ProfileOptions>::ConstIterator it = m_profiles.begin(); it != m_profiles.end(); ++it )
	{
		config->setGroup( it.key() );

		it.data().writeToConfig( config );
	}

	m_configMutex.unlock();

	delete config;

	return System::isWritable( path );
}
