/***************************************************************************
*   Copyright (C) 2005 by Sergio Pistone                                  *
*   sergio_pistone@yahoo.com.ar                                           *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "programsettings.h"
#include <transkodepluginsmanager.h>

#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

#include <kdialog.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include <kfiledialog.h>

ProgramSettings::ProgramSettings( QWidget* parent,  const char* name ):
	SettingsWidget( parent, name )
{
	if ( !name )
		setName( "ProgramSettings" );

	setCaption( i18n( "Program Settings" ) );

	QStringList programs = TransKodePluginsManager::programs();
	programs.sort();

	QGridLayout* mainLayout = new QGridLayout( this, 1, 1, programs.size(), 6, "ProgramSettingsLayout" );

	QGroupBox* groupBox = new QGroupBox( this, "groupBox1" );
	groupBox->setColumnLayout(0, Qt::Vertical );
	groupBox->layout()->setSpacing( 6 );
	groupBox->layout()->setMargin( 11 );
	QGridLayout* groupBoxLayout = new QGridLayout( groupBox->layout() );
	groupBoxLayout->setAlignment( Qt::AlignTop );

	groupBox->setTitle( i18n( "Programs' Paths" ) );

	QWidget* last = 0;
	int row = 0;
	for ( QStringList::ConstIterator it = programs.begin(); it != programs.end(); ++it, ++row )
	{
		QLabel* programLabel = new QLabel( groupBox );
		programLabel->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)5, 0, 0, programLabel->sizePolicy().hasHeightForWidth() ) );
		programLabel->setMinimumSize( QSize( 60, 26 ) );
		programLabel->setMaximumSize( QSize( 75, 32767 ) );
		programLabel->setText( "<b>" + *it + "</b>" );
		groupBoxLayout->addWidget( programLabel, row, 0 );
		m_programLabels[*it] = programLabel;

		QLineEdit* programPathLineEdit = new QLineEdit( groupBox );
		groupBoxLayout->addWidget( programPathLineEdit, row, 1 );
		connect( programPathLineEdit, SIGNAL( textChanged(const QString&) ), this, SLOT( emitSettingsChanged() ) );
		m_programPathLineEdits[*it] = programPathLineEdit;

		if ( last != 0 ) setTabOrder( last, programPathLineEdit );
		
		QPushButton* selectProgramButton = new QPushButton( groupBox );
		selectProgramButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, selectProgramButton->sizePolicy().hasHeightForWidth() ) );
		selectProgramButton->setMinimumSize( QSize( 40, 23 ) );
		selectProgramButton->setMaximumSize( QSize( 40, 23 ) );
		selectProgramButton->setText( "..." );
		selectProgramButton->installEventFilter( this );
		groupBoxLayout->addWidget( selectProgramButton, row, 2 );
		m_selectProgramButtons[*it] = selectProgramButton;

		setTabOrder( programPathLineEdit, selectProgramButton );
		
		QPushButton* guessProgramPathButton = new QPushButton( groupBox );
		guessProgramPathButton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, guessProgramPathButton->sizePolicy().hasHeightForWidth() ) );
		guessProgramPathButton->setMinimumSize( QSize( 40, 23 ) );
		guessProgramPathButton->setMaximumSize( QSize( 40, 23 ) );
		guessProgramPathButton->setText( i18n( "Guess" ) );
		//guessProgramPathButton->setAccel( QKeySequence( QString::null ) );
		groupBoxLayout->addWidget( guessProgramPathButton, row, 3 );
		guessProgramPathButton->installEventFilter( this );
		m_guessProgramPathButtons[*it] = guessProgramPathButton;

		setTabOrder( selectProgramButton, guessProgramPathButton );
		last = guessProgramPathButton;
	}

	mainLayout->addWidget( groupBox, 0, 0 );

	resize( QSize( 403, 591 ).expandedTo( minimumSizeHint() ) );

	clearWState( WState_Polished );
}

ProgramSettings::~ProgramSettings() {}

bool ProgramSettings::eventFilter( QObject* watched, QEvent* e )
{
	if ( e->type() == QEvent::MouseButtonRelease )
	{
		for ( QMap<QString,QPushButton*>::Iterator it = m_selectProgramButtons.begin(); it != m_selectProgramButtons.end(); ++it )
			if ( watched == it.data() )
				selectProgram( it.key() );

		for ( QMap<QString,QPushButton*>::Iterator it = m_guessProgramPathButtons.begin(); it != m_guessProgramPathButtons.end(); ++it )
			if ( watched == it.data() )
				guessProgramPath( it.key() );
	}

	return SettingsWidget::eventFilter( watched, e );
}


void ProgramSettings::selectProgram( const QString& program )
{
	QLineEdit* lineEdit = programPathLineEdit( program );
	if ( lineEdit == 0 )
		return;

	QString programPath = KFileDialog::getOpenFileName(
		lineEdit->text(),									// start path
		"",													// filter
		0,													// parent
		i18n ( "Find the path to %1" ).arg( program ) );	// caption

	if ( programPath != QString::null )
		lineEdit->setText( programPath );
}

void ProgramSettings::guessProgramPath( const QString& program )
{
	QLineEdit* lineEdit = programPathLineEdit( program );
	if ( lineEdit == 0 )
		return;

	QString programPath = KStandardDirs::findExe( program );
	if ( programPath != QString::null )
		lineEdit->setText( programPath );
}

QLineEdit* ProgramSettings::programPathLineEdit( const QString& program )
{
	return m_programPathLineEdits.contains( program ) ? m_programPathLineEdits[program] : 0;
}

QPushButton* ProgramSettings::guessProgramPathButton( const QString& program )
{
	return m_guessProgramPathButtons.contains( program ) ? m_guessProgramPathButtons[program] : 0;
}

QPushButton* ProgramSettings::selectProgramButton( const QString& program )
{
	return m_selectProgramButtons.contains( program ) ? m_selectProgramButtons[program] : 0;
}
