/***************************************************************************
 *   Copyright (C) 2005 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "transkodedialog.h"
#include <transkodepluginsmanager.h>

#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qgroupbox.h>
#include <qtooltip.h>
#include <qscrollview.h>
#include <qlayout.h>

#include <kmessagebox.h>
#include <kiconloader.h>
#include <klocale.h>

TransKodeDialog::TransKodeDialog( const QString& defaultsPath, bool script, QWidget* parent, const char* name ):
	KDialogBase::KDialogBase(	KDialogBase::IconList,	// dialogFace
								0,						// flags
								parent,					// parent
								name,					// name
								false,					// modal
								"Configure",			// caption
								Ok|Apply|Cancel|Default,// buttonMask
								Ok, 					// defaultButton
								true ),					// separator
	amarokScript( script ),
	acceptedConfig(),
	dirtyConfig(),
	defaultConfig( defaultsPath ),
	selectedProfile( "" )
{
	showButton( Default, true );

	KIconLoader iconLoader;

	addPage( generalSettings = new GeneralSettings(), "General", iconLoader.loadIcon( "misc", KIcon::Desktop, KIcon::SizeMedium ), "General Settings" );
	addPage( profileSettings = new ProfileSettings(), "Profiles", iconLoader.loadIcon( "favorites", KIcon::Desktop, KIcon::SizeMedium ), "Profiles Settings" );

//	addPage( programSettings = new ProgramSettings(), "Programs", iconLoader.loadIcon( "exec", KIcon::Desktop, KIcon::SizeMedium ), "Program Settings" );

	QScrollView* programSettingsSV = new QScrollView( 0, "ProgramSettingsScrollView" );
	programSettingsSV->setResizePolicy( QScrollView::AutoOneFit );
	QBoxLayout* layout = new QBoxLayout( QBoxLayout::TopToBottom );
	layout->addWidget( programSettingsSV );
	programSettings = new ProgramSettings( programSettingsSV->viewport() );
	programSettingsSV->addChild( programSettings );

	addPage( programSettingsSV, "Programs", iconLoader.loadIcon( "exec", KIcon::Desktop, KIcon::SizeMedium ), "Program Settings" );

	addPage( tagSettings = new TagSettings(), "Tags", iconLoader.loadIcon( "rss_tag", KIcon::Desktop, KIcon::SizeMedium ), "Tag Settings" );
// 	if ( amarokScript )
// 		addPage( streamSettings = new StreamSettings(), "Streams", iconLoader.loadIcon( "irkick", KIcon::Desktop, KIcon::SizeMedium ), "Streams Settings" );

	connect( this, SIGNAL( applyClicked(void) ), this, SLOT( acceptConfig(void) ) );
	connect( this, SIGNAL( okClicked(void) ), this, SLOT( acceptConfigAndClose(void) ) );
	connect( this, SIGNAL( cancelClicked(void) ), this, SLOT( rejectConfigAndClose(void) ) );
	connect( this, SIGNAL( defaultClicked(void) ), this, SLOT( setControlsFromDefaults(void) ) );
	connect( this, SIGNAL( defaultClicked(void) ), this, SLOT( enableApply(void) ) );

	setControlsFromConfig();

	// Apply button connections/setup
	connect( this, SIGNAL( defaultClicked(void) ), this, SLOT( enableApply(void) ) );
	connect( generalSettings, SIGNAL( settingsChanged() ), this, SLOT( enableApply(void) ) );
	connect( programSettings, SIGNAL( settingsChanged() ), this, SLOT( enableApply(void) ) );
	connect( tagSettings, SIGNAL( settingsChanged() ), this, SLOT( enableApply(void) ) );
	connect( profileSettings, SIGNAL( settingsChanged() ), this, SLOT( enableApply(void) ) );
	//if ( amarokScript )
	//	connect( streamSettings, SIGNAL( settingsChanged() ), this, SLOT( enableApply(void) ) );


	// Profiles setttings connections/setup
	connect( profileSettings->defaultNamingSchemeLineEdit, SIGNAL( textChanged(const QString&) ), this, SLOT( setProfileStatus(void) ) );
	connect( profileSettings->saveProfileButton, SIGNAL( clicked(void) ), this, SLOT( saveSelectedProfile(void) ) );
	connect( profileSettings->removeProfileButton, SIGNAL( clicked(void) ), this, SLOT( removeSelectedProfile(void) ) );
	connect( profileSettings->selectedProfileComboBox, SIGNAL( activated(int) ), this, SLOT( changeSelectedProfile(void) ) );

	changeSelectedProfile();
	enableButtonApply( false );

	if ( !amarokScript ) // this widgets only make sense for amarokScript
	{
		QToolTip::add(
			profileSettings->placeInMenuCheckBox, i18n(
			"<qt>Add this profile to 'transKode' entry in amaroK's playlist context menu."
			"<br/>Note: invalid profiles are <i>never</i> showed in the menu.</qt>" ) );

		generalSettings->askConfirmationCheckBox->hide();
		generalSettings->notifyProgressCheckBox->hide();
	}
	else
	{
		QToolTip::add(
			profileSettings->placeInMenuCheckBox, i18n(
			"<qt>Add this profile to 'transKode' context menu."
			"<br/>Note: invalid profiles are <i>never</i> showed in the menu.</qt>" ) );

		generalSettings->showTrayCheckBox->hide();

		fixedProfiles.append( "amaroK TTMD - m4a" );
		fixedProfiles.append( "amaroK TTMD - mp3" );
		fixedProfiles.append( "amaroK TTMD - ogg" );
	}

	profileSettings->encodingTypeComboBox->insertItem( "Bypass" );
	QStringList pluginNames = TransKodePluginsManager::encoderPluginNames();
	for ( QStringList::Iterator it = pluginNames.begin(); it != pluginNames.end(); ++it )
		profileSettings->encodingTypeComboBox->insertItem( *it );
}

TransKodeDialog::~TransKodeDialog()
{
}

const TransKodeConfig& TransKodeDialog::getConfig() const
{
	return acceptedConfig;
}

void TransKodeDialog::setConfig( const TransKodeConfig& cfg )
{
	bool visible = isVisible();

	if ( visible )
		hide();

	acceptedConfig = dirtyConfig = cfg;
	setControlsFromConfig();

	enableButtonApply( false );

	if ( visible )
		show();
}

void TransKodeDialog::saveSelectedProfile()
{
	QString oldProfile = "";
	QString newProfile = profileSettings->selectedProfileComboBox->lineEdit()->text();
	if ( newProfile == "" )
		return;

	// Alert if newProfile colides with an existing one
	int idx = -1;
	if ( profileSettings->selectedProfileComboBox->count() )
		for ( idx = 0; idx < profileSettings->selectedProfileComboBox->count(); ++idx )
		{
			oldProfile = profileSettings->selectedProfileComboBox->text( idx );
			if ( oldProfile == newProfile )
			{
				if ( KMessageBox::warningContinueCancel(
						this,
						i18n( "<qt><p align=\"center\">Do you want to overwrite it?</p></qt>" ),
						i18n( "Profile Already Exists" ) ) == KMessageBox::Cancel )
					return;
				break;
			}
		}

	ProfileOptions profileOptions;

	profileOptions.setNamingScheme( profileSettings->namingSchemeLineEdit->text() );
	profileOptions.setSwitches( profileSettings->encodingSwitchesLineEdit->text() );
	profileOptions.setShow( profileSettings->placeInMenuCheckBox->isChecked() );
	if ( !fixedProfiles.contains( newProfile ) )
	{
		QLineEdit* lineEdit = profileSettings->outputExtensionComboBox->lineEdit();
		profileOptions.setExtension( lineEdit ? lineEdit->text() : profileSettings->outputExtensionComboBox->currentText() );
		profileOptions.setEncodeType( profileSettings->encodingTypeComboBox->currentText() );
	}
	else
	{
		ProfileOptions* fixedOptions = dirtyConfig.profileOptions( newProfile );
		if ( fixedOptions != 0 )
		{
			profileOptions.setExtension( fixedOptions->extension() );
			profileOptions.setEncodeType( fixedOptions->encodeType() );
			delete fixedOptions;
		}
		else // this should never happen!!!
			return;

		QString selectedItemProfile = profileSettings->selectedProfileComboBox->text(
			profileSettings->selectedProfileComboBox->currentItem() );
		if ( selectedItemProfile != newProfile )
		{
			KMessageBox::information(
				this,
				i18n( "<qt>Encode type and extension can't be changed for this profile</qt>" ),
				i18n( "Special Profile" ),
				"fixed profiles" );
			// if we get here, idx contains the index of the profile we are overwriting
			profileSettings->selectedProfileComboBox->setCurrentItem( idx );
		}
	}

	dirtyConfig.setProfileOptions( newProfile, profileOptions );

	if ( oldProfile != newProfile ) // profile didn't exist before
	{
		profileSettings->selectedProfileComboBox->insertItem( newProfile );
		profileSettings->selectedProfileComboBox->setCurrentItem( profileSettings->selectedProfileComboBox->count()-1 );
	}

	changeSelectedProfile();
}

void TransKodeDialog::removeSelectedProfile()
{
	if ( profileSettings->selectedProfileComboBox->count() )
	{
		int currentIdx = profileSettings->selectedProfileComboBox->currentItem();
		QString oldProfile = profileSettings->selectedProfileComboBox->text( currentIdx );
		QString newProfile = profileSettings->selectedProfileComboBox->lineEdit()->text();
		if ( oldProfile != newProfile )
			return;

		profileSettings->selectedProfileComboBox->removeItem( currentIdx );
		dirtyConfig.removeProfile( oldProfile );

		if ( profileSettings->selectedProfileComboBox->count() )
		{
			profileSettings->selectedProfileComboBox->setCurrentItem( currentIdx != 0 ? currentIdx - 1 : 0 );
			changeSelectedProfile(); // It's not automatically called
		}
		else
			profileSettings->profileStatusLabel->setText( "" );
	}
}

void TransKodeDialog::changeSelectedProfile()
{
	selectedProfile = profileSettings->selectedProfileComboBox->currentText();
	if ( selectedProfile.isEmpty() )
	{
		QStringList pluginNames = TransKodePluginsManager::encoderPluginNames();
		if ( pluginNames.size() > 0 )
		{
			profileSettings->changeEncodingType( pluginNames.first() );
			profileSettings->profileStatusLabel->setText( "" );
		}
		return;
	}

	ProfileOptions* profileOptions = dirtyConfig.profileOptions( selectedProfile );
	if ( !profileOptions )
		return;
	profileSettings->changeEncodingType( profileOptions->encodeType() );
	profileSettings->namingSchemeLineEdit->setText( profileOptions->displayNamingScheme() );
	profileSettings->encodingSwitchesLineEdit->setText( profileOptions->switches() );
	QLineEdit* lineEdit = profileSettings->outputExtensionComboBox->lineEdit();
	if ( lineEdit != 0 )
		lineEdit->setText( profileOptions->extension() );
	profileSettings->placeInMenuCheckBox->setChecked( profileOptions->show() );
	delete profileOptions;

	bool fixed = fixedProfiles.contains( selectedProfile );
	profileSettings->encodingTypeComboBox->setDisabled( fixed );
	profileSettings->outputExtensionComboBox->setDisabled( fixed );
	profileSettings->removeProfileButton->setDisabled( fixed );
// 	profileSettings->encodingTypeComboBox->setDisabled( fixed ? true : !profileSettings->encodingTypeComboBox->isEnabled() );
// 	profileSettings->outputExtensionComboBox->setDisabled( fixed ? true : !profileSettings->outputExtensionComboBox->isEnabled() );
// 	profileSettings->removeProfileButton->setDisabled( fixed ? true : !profileSettings->removeProfileButton->isEnabled() );

	setProfileStatus();
}

void TransKodeDialog::setProfileStatus()
{
	if ( selectedProfile.isEmpty() )
	{
		profileSettings->profileStatusLabel->setText( "" );
		return;
	}

	QString status = "";

	ProfileOptions* profileOpts = acceptedConfig.profileOptions( selectedProfile );
	if ( !profileOpts )
		return;

	if ( ( acceptedConfig.generalOpt( GeneralOpt::naming_scheme ) != profileSettings->defaultNamingSchemeLineEdit->text()
		   && profileOpts->usesDefaultNamingScheme() ) )
		status += i18n( "<font color=\"#ffff00\"><b>Modified defaults prevent validation of this profile "
						"(apply settings or undo changes to remove this warning).</b></font>" );
	else
	{
		if ( ! acceptedConfig.encodeTypeIsSupported( profileOpts->encodeType() ) )
			status += i18n( "<font color=\"#ff0000\"><b>Encoding type is not supported</b></font>" );

		if ( ! profileOpts->haveValidNamingScheme() )
			status += (status.isEmpty() ? QString("") : QString("<br/>")) + i18n( "<font color=\"#ff0000\"><b>Naming scheme is invalid</b></font>" );

		if ( ! status.isEmpty() )
			status = "<qt>" + status + "</qt>";
		else
			status = i18n( "<qt>Profile settings are valid</qt>" );
	}

	delete profileOpts;

	profileSettings->profileStatusLabel->setText( status );
}

void TransKodeDialog::setControlsFromConfig()
{
	generalSettings->overwriteCheckBox->setChecked( dirtyConfig.generalOpt( GeneralOpt::overwrite ) == "true" ? true:false );
	generalSettings->caseInsensitiveCheckBox->setChecked( dirtyConfig.generalOpt( GeneralOpt::case_insensitive ) == "true" ? true:false );
	generalSettings->replaceCharsCheckBox->setChecked( dirtyConfig.generalOpt( GeneralOpt::replace_chars ).isEmpty() ? false:true );

	generalSettings->notifyProgressCheckBox->setChecked( dirtyConfig.generalOpt( GeneralOpt::notify ) == "true" ? true:false );
	generalSettings->askConfirmationCheckBox->setChecked( dirtyConfig.generalOpt( GeneralOpt::confirm ) == "true" ? true:false );

	generalSettings->showTrayCheckBox->setChecked( dirtyConfig.generalOpt( GeneralOpt::show_tray ) == "true" );

	generalSettings->logEventsCheckBox->setChecked( ! dirtyConfig.generalOpt( GeneralOpt::log ).isEmpty() );
	if ( generalSettings->logEventsCheckBox->isChecked() )
		generalSettings->logEventsLineEdit->setText( dirtyConfig.generalOpt( GeneralOpt::log ) );

	generalSettings->tempDirLineEdit->setText( dirtyConfig.generalOpt( GeneralOpt::temp_dir ) );

	generalSettings->prioritySlider->setValue( dirtyConfig.generalOpt( GeneralOpt::proc_priority ).toInt() );

	tagSettings->artistComboBox->setCurrentItem( dirtyConfig.tagCase( Tag::artist ) );
	tagSettings->albumComboBox->setCurrentItem( dirtyConfig.tagCase( Tag::album ) );
	tagSettings->titleComboBox->setCurrentItem( dirtyConfig.tagCase( Tag::title ) );
	tagSettings->commentComboBox->setCurrentItem( dirtyConfig.tagCase( Tag::comment ) );
	tagSettings->genreComboBox->setCurrentItem( dirtyConfig.tagCase( Tag::genre ) );

	tagSettings->artistLineEdit->setText( dirtyConfig.tagDefault( Tag::artist ) );
	tagSettings->yearLineEdit->setText( dirtyConfig.tagDefault( Tag::year ) );
	tagSettings->albumLineEdit->setText( dirtyConfig.tagDefault( Tag::album ) );
	tagSettings->trackNumberLineEdit->setText( dirtyConfig.tagDefault( Tag::track ) );
	tagSettings->titleLineEdit->setText( dirtyConfig.tagDefault( Tag::title ) );
	tagSettings->commentLineEdit->setText( dirtyConfig.tagDefault( Tag::comment ) );
	tagSettings->genreLineEdit->setText( dirtyConfig.tagDefault( Tag::genre ) );

	QStringList programs = TransKodePluginsManager::programs();
	for ( QStringList::Iterator it = programs.begin(); it != programs.end(); ++it )
	{
		QLineEdit* lineEdit = programSettings->programPathLineEdit( *it );
		if ( lineEdit != 0 )
			lineEdit->setText( TransKodePluginsManager::programPath( *it ) );
	}

// 	if ( amarokScript )
// 	{
// 		bool isNumber;
// 		int ripDuration = dirtyConfig.generalOpt( GeneralOpt::rip_duration ).toInt( &isNumber );
// 		if ( !isNumber )
// 			ripDuration = 600;
// 		streamSettings->ripDurationSpinBox->setValue( ripDuration );
// 		streamSettings->ripIncompleteCheckBox->setChecked( dirtyConfig.generalOpt( GeneralOpt::rip_incomplete ) == "true" ? true:false );
// 	}

	profileSettings->defaultNamingSchemeLineEdit->setText( dirtyConfig.generalOpt( GeneralOpt::naming_scheme ) );

	// Add profiles to selectedProfileComboBox
	profileSettings->selectedProfileComboBox->clear();
	QStringList profilesList = dirtyConfig.profiles();
	for ( QStringList::Iterator it = profilesList.begin(); it != profilesList.end(); ++it )
		profileSettings->selectedProfileComboBox->insertItem( *it );

	if ( profileSettings->selectedProfileComboBox->count() )
	{
		profileSettings->selectedProfileComboBox->setCurrentItem( 0 );
		changeSelectedProfile();
	}
}

void TransKodeDialog::setControlsFromDefaults()
{
	int activeIdx = activePageIndex();
	if ( activeIdx == 0 )
	{
		setConfigFromControls();

		dirtyConfig.setGeneralOpt( GeneralOpt::overwrite, defaultConfig.generalOpt( GeneralOpt::overwrite ) );
		dirtyConfig.setGeneralOpt( GeneralOpt::case_insensitive, defaultConfig.generalOpt( GeneralOpt::case_insensitive ) );
		dirtyConfig.setGeneralOpt( GeneralOpt::replace_chars, defaultConfig.generalOpt( GeneralOpt::replace_chars ) );
		dirtyConfig.setGeneralOpt( GeneralOpt::notify, defaultConfig.generalOpt( GeneralOpt::notify ) );
		dirtyConfig.setGeneralOpt( GeneralOpt::confirm, defaultConfig.generalOpt( GeneralOpt::confirm ) );
		dirtyConfig.setGeneralOpt( GeneralOpt::show_tray, defaultConfig.generalOpt( GeneralOpt::show_tray ) );
		dirtyConfig.setGeneralOpt( GeneralOpt::log, defaultConfig.generalOpt( GeneralOpt::log ) );
		dirtyConfig.setGeneralOpt( GeneralOpt::temp_dir, defaultConfig.generalOpt( GeneralOpt::temp_dir ) );
	}
	else if ( activeIdx == 1 )
	{
		setConfigFromControls();

		dirtyConfig.setGeneralOpt( GeneralOpt::naming_scheme, defaultConfig.generalOpt( GeneralOpt::naming_scheme ) );

		dirtyConfig.clearProfiles();

		QStringList profiles = defaultConfig.profiles();
		for ( QStringList::Iterator it = profiles.begin(); it != profiles.end(); ++it )
		{
			ProfileOptions* profileOpts = defaultConfig.profileOptions( *it );
			if ( !profileOpts )
				continue;
			dirtyConfig.setProfileOptions( *it, *profileOpts );
			delete profileOpts;
		}

		setProfileStatus();
	}
	else if ( activeIdx == 2 )
	{
		setConfigFromControls();

		QStringList programs = TransKodePluginsManager::programs();
		for ( QStringList::Iterator it = programs.begin(); it != programs.end(); ++it )
		{
			QLineEdit* lineEdit = programSettings->programPathLineEdit( *it );
			if ( lineEdit != 0 )
				lineEdit->setText( TransKodePluginsManager::programPath( *it ) );
		}

		return;
	}
	else if ( activeIdx == 3 )
	{
		setConfigFromControls();

		for ( int idx = 0; idx < Tag::SIZE; ++idx )
			dirtyConfig.setTagCase( (Tag::TTag)idx, defaultConfig.tagCase( (Tag::TTag)idx ) );
		for ( int idx = 0; idx < Tag::SIZE; ++idx )
			dirtyConfig.setTagDefault( (Tag::TTag)idx, defaultConfig.tagDefault( (Tag::TTag)idx ) );
	}
	else if ( activeIdx == 4 )
	{
		setConfigFromControls();

		dirtyConfig.setGeneralOpt( GeneralOpt::rip_duration, defaultConfig.generalOpt( GeneralOpt::rip_duration ) );
		dirtyConfig.setGeneralOpt( GeneralOpt::rip_incomplete, defaultConfig.generalOpt( GeneralOpt::rip_incomplete ) );
	}
	else
		return;

	setControlsFromConfig();
}

void TransKodeDialog::setConfigFromControls()
{
	dirtyConfig.setGeneralOpt( GeneralOpt::overwrite, generalSettings->overwriteCheckBox->isChecked() ? "true" : "false" );
	dirtyConfig.setGeneralOpt( GeneralOpt::case_insensitive, generalSettings->caseInsensitiveCheckBox->isChecked() ? "true" : "false" );
	dirtyConfig.setGeneralOpt( GeneralOpt::replace_chars,	generalSettings->replaceCharsCheckBox->isChecked() ?
															dirtyConfig.generalOpt( GeneralOpt::replace_chars ) : "" );

	dirtyConfig.setGeneralOpt( GeneralOpt::notify, generalSettings->notifyProgressCheckBox->isChecked() ? "true" : "false" );
	dirtyConfig.setGeneralOpt( GeneralOpt::confirm, generalSettings->askConfirmationCheckBox->isChecked() ? "true" : "false" );

	dirtyConfig.setGeneralOpt( GeneralOpt::show_tray, generalSettings->showTrayCheckBox->isChecked() ? "true" : "false" );

	dirtyConfig.setGeneralOpt( GeneralOpt::proc_priority, QString::number( generalSettings->prioritySlider->value() ) );

	dirtyConfig.setGeneralOpt( GeneralOpt::log, generalSettings->logEventsCheckBox->isChecked() ? generalSettings->logEventsLineEdit->text() : "" );

	dirtyConfig.setGeneralOpt( GeneralOpt::temp_dir, generalSettings->tempDirLineEdit->text() );

// 	if ( amarokScript )
// 	{
// 		dirtyConfig.setGeneralOpt( GeneralOpt::rip_incomplete, streamSettings->ripIncompleteCheckBox->isChecked() ? "true" : "false" );
// 		dirtyConfig.setGeneralOpt( GeneralOpt::rip_duration, QString::number( streamSettings->ripDurationSpinBox->value() ) );
// 	}

	dirtyConfig.setGeneralOpt( GeneralOpt::naming_scheme, profileSettings->defaultNamingSchemeLineEdit->text() );

	dirtyConfig.setTagCase( Tag::artist, (CaseFunc::TCaseFunc)tagSettings->artistComboBox->currentItem() );
	dirtyConfig.setTagCase( Tag::album, (CaseFunc::TCaseFunc)tagSettings->albumComboBox->currentItem() );
	dirtyConfig.setTagCase( Tag::title, (CaseFunc::TCaseFunc)tagSettings->titleComboBox->currentItem() );
	dirtyConfig.setTagCase( Tag::comment, (CaseFunc::TCaseFunc)tagSettings->commentComboBox->currentItem() );
	dirtyConfig.setTagCase( Tag::genre, (CaseFunc::TCaseFunc)tagSettings->genreComboBox->currentItem() );

	dirtyConfig.setTagDefault( Tag::artist, tagSettings->artistLineEdit->text() );
	dirtyConfig.setTagDefault( Tag::year, tagSettings->yearLineEdit->text() );
	dirtyConfig.setTagDefault( Tag::album, tagSettings->albumLineEdit->text() );
	dirtyConfig.setTagDefault( Tag::track, tagSettings->trackNumberLineEdit->text() );
	dirtyConfig.setTagDefault( Tag::title, tagSettings->titleLineEdit->text() );
	dirtyConfig.setTagDefault( Tag::comment, tagSettings->commentLineEdit->text() );
	dirtyConfig.setTagDefault( Tag::genre, tagSettings->genreLineEdit->text() );

	QStringList programs = TransKodePluginsManager::programs();
	for ( QStringList::Iterator it = programs.begin(); it != programs.end(); ++it )
	{
		QLineEdit* lineEdit = programSettings->programPathLineEdit( *it );
		if ( lineEdit != 0 )
			dirtyConfig.setProgramPath( *it, lineEdit->text() );
	}
}


void TransKodeDialog::acceptConfig()
{
	setConfigFromControls();
	acceptedConfig = dirtyConfig;

	changeSelectedProfile();
	enableButtonApply( false );

	emit configSaved();
}

void TransKodeDialog::acceptConfigAndClose()
{
	acceptConfig();
	hide();
}

void TransKodeDialog::rejectConfig()
{
	dirtyConfig = acceptedConfig;
	setControlsFromConfig();
	enableButtonApply( false );
}

void TransKodeDialog::rejectConfigAndClose()
{
	hide();
	rejectConfig();
}

void TransKodeDialog::addPage( QWidget* page, const QString& itemName, const QPixmap& pixmap, const QString& header )
{
	QVBox *frame = addVBoxPage( itemName, header, pixmap );
	frame->setSpacing( 0 );
	frame->setMargin( 0 );
	page->reparent( (QWidget*)frame, 0, QPoint() );
}

void TransKodeDialog::addPage( QWidget* page, const QString& itemName, const QString& pixmapName, const QString& header )
{
	QVBox *frame = addVBoxPage( itemName, header, pixmapName );
	frame->setSpacing( 0 );
	frame->setMargin( 0 );
	page->reparent( (QWidget*)frame, 0, QPoint() );
}

void TransKodeDialog::enableApply()
{
	enableButtonApply( true );
}
