/***************************************************************************
 *   Copyright (C) 2005 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "processoutput.h"
#include <commondefs.h>

OutputParser::OutputParser( QObject* parent, const char* name ):
	QObject( parent, name ) {};

OutputParser::~OutputParser() {}

OutputGrabber::OutputGrabber( QProcess* process ):
	m_process( process )
{
	if ( m_process == 0 )
		return;

	int comm = m_process->communication();
	if ( comm & QProcess::Stdout )
		connect( m_process, SIGNAL( readyReadStdout() ), this, SLOT( saveStdout() ) );
	if ( comm & QProcess::Stderr && !(comm & QProcess::DupStderr) )
		connect( m_process, SIGNAL( readyReadStderr() ), this, SLOT( saveStderr() ) );
}

OutputGrabber::~OutputGrabber() { disconnect(); }

QString OutputGrabber::getOutput()
{
	m_mutex.lock();

	QString ret = String::deepCopy( m_output );

	m_mutex.unlock();

	return ret;
}

QString OutputGrabber::takeOutput()
{
	m_mutex.lock();

	QString ret = m_output;
	m_output = QString::null;

	m_mutex.unlock();

	return ret;
}

void OutputGrabber::saveStdout()
{
	QString output = m_process->readStdout();

	LOG( "PROCESS PARSER STDOUT: " + output, PLUGIN );

	m_mutex.lock();

	m_output += output;

	m_mutex.unlock();
}

void OutputGrabber::saveStderr()
{
	QString output = m_process->readStderr();

	LOG( "PROCESS PARSER STDERR: " + output, PLUGIN );

	m_mutex.lock();

	m_output += output;

	m_mutex.unlock();
}


OutputParserProxy::OutputParserProxy( QProcess* process, OutputParser* parser, unsigned pollInterval ):
	QThread(),
	m_parser( parser ),
	m_pollInterval( pollInterval ),
	m_outputGrabber( process ),
	m_stop( false ),
	m_mutex( false )
{
	if ( process == 0 || m_parser == 0 )
		return;

	LOG( "STARTING PROCESS OUTPUT PARSER THREAD", PLUGIN );
	start();
}

void OutputParserProxy::run()
{
	while ( ! stop() )
	{
		QString output = m_outputGrabber.takeOutput();
		if ( ! output.isEmpty() )
			m_parser->parseOutput( output );

		QThread::msleep( m_pollInterval );
	}

	QString output = m_outputGrabber.takeOutput();
	while ( ! output.isEmpty() )
	{
		m_parser->parseOutput( output );
		output = m_outputGrabber.takeOutput();
	}

	m_stoppedCondition.wakeAll();
}

bool OutputParserProxy::stop()
{
	m_mutex.lock();

	bool ret = m_stop;

	m_mutex.unlock();

	return ret;
}

OutputParserProxy::~OutputParserProxy()
{
	LOG( "DELETING PROCESS OUTPUT PARSER", PLUGIN );

	if ( m_parser != 0 )
	{
		m_outputGrabber.disconnect();

		m_mutex.lock();

		m_stop = true;

		m_stoppedCondition.wait( &m_mutex );

		m_mutex.unlock();

		m_parser->disconnect();

		//delete m_parser;
		m_parser->deleteLater();
	}
}
