/***************************************************************************
 *   Copyright (C) 2005 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include "qprocesswrapper.h"
#include <commondefs.h>

#include <qtimer.h>
#include <qregexp.h>

QProcessWrapper::QProcessWrapper( int priority, int comm, QObject* parent, const char* name ):
	QProcess( parent, name ),
	m_priority( priority ),
	m_paused( false ),
	m_shellParsing( false ) { setCommunication( comm ); }

QProcessWrapper::QProcessWrapper( const QString& arg0, int priority, int comm, QObject* parent, const char* name ):
	QProcess( arg0, parent, name ),
	m_priority( priority ),
	m_paused( false ),
	m_shellParsing( false ) { setCommunication( comm ); }

QProcessWrapper::QProcessWrapper( const QStringList& args, int priority, int comm, QObject* parent, const char* name ):
	QProcess( args, parent, name ),
	m_priority( priority ),
	m_paused( false ),
	m_shellParsing( false ) { setCommunication( comm ); }

QProcessWrapper::~QProcessWrapper() { kill(); }

bool QProcessWrapper::start( QStringList* env )
{
	bool ret = QProcess::start( env );

	if ( ret )
	{
		if ( !setPriority( m_priority ) )
			m_priority = 0;
	}
	else
		m_priority = 0;

	return ret;
}

void QProcessWrapper::terminate( unsigned msecs )
{
	if ( msecs > 0 )
	{
		tryTerminate();
		QTimer::singleShot( msecs, this, SLOT( kill() ) );
	}
	else
		kill();
}

bool QProcessWrapper::waitProcess( unsigned msecs )
{
	unsigned waited = 0;
	while ( waited < msecs )
	{
		if ( !isRunning() )
			return true;
		m_waitCondition.wait( 10 );
		waited += 10;
	}
	return !isRunning();
}

int QProcessWrapper::priority() const
{
	return m_priority;
}


bool QProcessWrapper::setPriority( int priority )
{
#ifdef Q_OS_UNIX
	if ( !isRunning() )
	{
		m_priority = priority;
		return false;
	}

	if ( m_priority != priority )
	{
		int pid = processIdentifier();
		int ret = ::system( QString( "renice %1 -p %2" ).arg( priority ).arg( pid ) );
		if ( ret == 0 )
			m_priority = priority;
	}
	return m_priority == priority;
#else
	return m_priority == priority;
#endif
}

bool QProcessWrapper::signal( Signal sig )
{
	#ifdef Q_OS_UNIX
		QString sigName;
		switch ( sig )
		{
			case POSIX_SIGABRT:		sigName = "ABRT";		break;
			case POSIX_SIGALRM:		sigName = "ALRM";		break;
			case POSIX_SIGFPE:		sigName = "FPE";		break;
			case POSIX_SIGHUP:		sigName = "HUP";		break;
			case POSIX_SIGILL:		sigName = "ILL";		break;
			case POSIX_SIGINT:		sigName = "INT";		break;
			case POSIX_SIGKILL:		sigName = "KILL";		break;
			case POSIX_SIGPIPE:		sigName = "PIPE";		break;
			case POSIX_SIGQUIT:		sigName = "QUIT";		break;
			case POSIX_SIGSEGV:		sigName = "SEGV";		break;
			case POSIX_SIGTERM:		sigName = "TERM";		break;
			case POSIX_SIGUSR1:		sigName = "USR1";		break;
			case POSIX_SIGUSR2:		sigName = "USR2";		break;
			case POSIX_SIGCHLD:		sigName = "CHLD";		break;
			case POSIX_SIGCONT:		sigName = "CONT";		break;
			case POSIX_SIGSTOP:		sigName = "STOP";		break;
			case POSIX_SIGTSTP:		sigName = "TSTP";		break;
			case POSIX_SIGTTIN:		sigName = "TTIN";		break;
			case POSIX_SIGTTOU:		sigName = "TTOU";		break;
			default:						return false;
		}

		int pid = processIdentifier();
		LOG( QString( "kill -s %1 %2" ).arg( sigName ).arg( pid ), WORKER );
		return ::system( QString( "kill -s %1 %2" ).arg( sigName ).arg( pid ) ) == 0;
	#else
		return false;
	#endif
}

bool QProcessWrapper::paused() const
{
	return m_paused;
}

bool QProcessWrapper::pause()
{
#ifdef Q_OS_UNIX
	if ( !m_paused )
		if ( signal( POSIX_SIGSTOP ) )
			m_paused = true;
	return m_paused;
#else
	return m_paused;
#endif
}

bool QProcessWrapper::resume()
{
#ifdef Q_OS_UNIX
	if ( m_paused )
		if ( signal( POSIX_SIGCONT ) )
			m_paused = false;
	return !m_paused;
#else
	return !m_paused;
#endif
}

void QProcessWrapper::setShellParsingMode( bool value )
{
	m_shellParsing = value;
}

bool QProcessWrapper::getShellParsingMode()
{
	return m_shellParsing;
}


void QProcessWrapper::setArguments( const QStringList& args )
{
	clearArguments();
	addArguments( args );
}

void QProcessWrapper::addArgument( const QString& arg )
{
	if ( m_shellParsing )
	{
		QStringList args = parseShellArgs( arg );
		for ( QStringList::ConstIterator it = args.begin(); it != args.end(); ++it )
			QProcess::addArgument( *it );
	}
	else
		QProcess::addArgument( arg );
}

void QProcessWrapper::addArguments( const QStringList& args )
{
	for ( QStringList::ConstIterator it = args.begin(); it != args.end(); ++it )
		addArgument( *it );
}

QProcessWrapper& QProcessWrapper::operator<<( const QStringList& args )
{
	addArguments( args );
	return *this;
}

QProcessWrapper& QProcessWrapper::operator<<( const char* arg )
{
	addArgument( arg );
	return *this;
}

QProcessWrapper& QProcessWrapper::operator<<( const QByteArray& arg )
{
	return operator<<( arg.data() );
}

QProcessWrapper& QProcessWrapper::operator<<( const QString& arg )
{
	addArgument( arg );
	return *this;
}


QStringList QProcessWrapper::parseShellArgs( const QString& args )
{
	#define FIND_ARG_START	0
	#define FIND_ARG_END	1

	static const QRegExp regExp( "\\\\([^\\\\])" );
	bool escaped = false;
	bool action = FIND_ARG_START;
	QChar argStartChar = '\0';
	unsigned argStartIdx = 0;
	unsigned argEndIdx = 0;

	QStringList ret;

	for( unsigned idx = 0; idx < args.length(); ++idx )
	{
		bool findArgEnd = action == FIND_ARG_END;

		const QChar c = args.at( idx );
		if ( action == FIND_ARG_START )
		{
			if (!escaped && ( c == '"' || c == '\'' ))
			{
				argStartChar = c;
				argStartIdx = idx;
				action = FIND_ARG_END;
			}
			else if ( (escaped && c == ' ') || (!escaped && c != ' ') )
			{
				argStartChar = '\0';
				argStartIdx = idx;
				action = FIND_ARG_END;
				if ( idx == args.length()-1 )
					findArgEnd = true;
			}
		}

		if ( findArgEnd )
		{
			if ( /*argStartChar != ' ' &&*/ !escaped && c == argStartChar )
			{
				action = FIND_ARG_START;
				QString arg = args.mid( argStartIdx + 1, idx - argStartIdx - 1 );
				if ( c == '"' )
					arg.replace( "\\\\", "\\" );
				if ( argEndIdx+1 == argStartIdx && ret.size() > 0 ) // join with last parameter
				{
					arg = ret.last() + arg;
					ret.pop_back();
				}
				ret.append( arg );
				argEndIdx = idx;
			}
			else if ( argStartChar == '\0' && ( (!escaped && c == ' ') || (idx == args.length()-1) ) )
			{
				action = FIND_ARG_START;
				QString arg = args.mid( argStartIdx, idx - argStartIdx - (c == ' ' ? 0 : -1) );
				arg.replace( "\\ ", " " );
				arg.replace( regExp /* QRegExp( "\\\\([^\\\\])" ) */, "\\1" );
				arg.replace( "\\\\", "\\" );
				if ( argEndIdx+1 == argStartIdx && ret.size() > 0 ) // join with last parameter
				{
					arg = ret.last() + arg;
					ret.pop_back();
				}
				ret.append( arg );
				argEndIdx = idx - 1;
			}
		}

		if ( escaped )
			escaped = false;
		else
			escaped = c == '\\';
	}

	return ret;
}
