/***************************************************************************
 *   Copyright (C) 2005 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "transkodescript.h"
#include <transkodepluginsmanager.h>

#include <signal.h>

#include <qregexp.h>

#include <kstandarddirs.h>
#include <klocale.h>
#include <kurl.h>
#include <dcopref.h>
#include <kmessagebox.h>
#include <kpassivepopup.h>

#define SHOW_POPUP (QEvent::Type)(QEvent::User + 1)

TransKodeScript::TransKodeScript():
	KApplication(),
	m_listener(),
	m_logger( QString::null ),
	m_config( ::locate( "data", "amarok/scripts/transkode/defaultsrc" ) ),
	m_configDialog( ::locate( "data", "amarok/scripts/transkode/defaultsrc" ), true ),
	m_fileWorkerThread( m_logger, m_config, true ),
	m_amarokWorkerThread( m_logger, m_config, true )
{
	signal( SIGTERM, cleanUp );

	m_config.load( ::locate( "data", "amarok/scripts-data/transkoderc" ) );
	m_configDialog.setConfig( m_config );
	m_logger.setFile( m_config.generalOpt( GeneralOpt::log ), true );
	m_logger.setMsgPrefix( "[script] " );

	TransKodePluginsManager::setLogger( &m_logger );

	// If there are no user defined settings show config dialog
	if ( !System::isReadable( ::locate( "data", "amarok/scripts-data/transkoderc" ) ) )
		slotPreferences();

	connect( &m_configDialog, SIGNAL( configSaved() ), this, SLOT( slotConfig() ) ) ;
	slotConfig();

	m_logger.log( "starting main worker thread" );
	m_fileWorkerThread.start();

	m_logger.log( "starting amaroK's TTMD dedicated worker thread" );
	m_amarokWorkerThread.start();

	m_jobs.setAutoDelete( true ); // only a precaution

	connect( &m_listener, SIGNAL( communicationClosed() ), this, SLOT( quit() ) );
	connect( &m_listener, SIGNAL( configureRequested() ), this, SLOT( slotPreferences() ) );
	connect( &m_listener, SIGNAL( transcodeRequested(QString) ), this, SLOT( addJobs(QString) ) );

	connect( &m_fileWorkerThread, SIGNAL( mtRoundStats(long,int,int,int,int,int,int) ),
		this, SLOT( displayRoundStats(long,int,int,int,int,int,int) ) );

	connect( &m_amarokWorkerThread, SIGNAL( mtRoundFinished(long) ),
		this, SLOT( amarokDone() ) );
}

void TransKodeScript::cleanUp( int ) { kapp->quit(); }
TransKodeScript::~TransKodeScript()
{
	setupMenuEntries( remove ); // Remove menu entries

	m_logger.log( "stopping main worker thread" );
	m_fileWorkerThread.resume();
	m_fileWorkerThread.stop();

	m_logger.log( "stopping amaroK's TTMD worker thread" );
	m_amarokWorkerThread.resume();
	m_amarokWorkerThread.stop();

	m_logger.log( "waiting all worker threads to end" );
	m_fileWorkerThread.wait();
	m_amarokWorkerThread.wait();

	m_logger.log( "worker threads finished" );
	m_logger.log( "exiting" );
}

void TransKodeScript::slotPreferences()
{
	QSize size = m_configDialog.size();
	m_configDialog.setInitialSize( size );
	m_configDialog.show();
}

void TransKodeScript::slotConfig()
{
	setupMenuEntries( remove ); // Remove old menu entries

	QString logFile = m_logger.file();

	QString oldTargetFormats = m_config.supportedEncodingExtensions();

	m_config = m_configDialog.getConfig();
	m_config.save( ::locateLocal( "data", "amarok/scripts-data/transkoderc" ) );

	m_logger.setFile( m_config.generalOpt( GeneralOpt::log ), true );

	QString newTargetFormats = m_config.supportedEncodingExtensions();

	if ( m_config.generalOpt( GeneralOpt::log ) != logFile )
	{
		m_logger.setFile( m_config.generalOpt( GeneralOpt::log ) );
		m_logger.truncate();
	}

	setupMenuEntries( add ); // Insert new menu entries

	if ( newTargetFormats != oldTargetFormats )
	{
		KConfig specsFile( ::locate( "data", "amarok/scripts/transkode/transkode.spec" ) );
		specsFile.setGroup( "Transcode" );
		specsFile.writeEntry( "target_formats", newTargetFormats );
	}
}

void TransKodeScript::setupMenuEntries( TAction action )
{
	QCString function = action == add ? "addCustomMenuItem(QString,QString)" : "removeCustomMenuItem(QString,QString)";
	DCOPRef amarokscript( "amarok", "script" );
	QStringList profiles = m_config.validProfiles();
	for ( QStringList::Iterator it = profiles.begin(); it != profiles.end(); ++it )
	{
		ProfileOptions* options = m_config.profileOptions( *it );
		if ( options == 0 )
			continue;
		if ( options->show() )
			amarokscript.send( function, QString( "transKode" ), QString( "[ " + *it + " ]" ) );
		delete options;
	}
}

void TransKodeScript::addJobs( QString message )
{
	if ( message.left( strlen( "transcode " ) ) == "transcode " ) // amaroK media device request
	{
		QStringList tokens = QStringList::split( " ", message );

		KURL srcURL( tokens[1] );
		QString profile = "amaroK TTMD - " + tokens[2].lower();

		if ( !m_config.profileIsValid( profile ) || srcURL.protocol() != "file" ) // can't handle anything else than local files ATM
		{
			amarokError( 0, srcURL.url() );
			return;
		}

		m_logger.log( "message received from amaroK' TTMD" );

		m_logger.log( "profile: " + profile );
		m_logger.log( "file: " + srcURL.path() );

		TransKodeFileJob* job = new TransKodeFileJob( m_config, srcURL.path() );
		job->setProfile( profile );
		m_jobs.append( job );

		connect( job, SIGNAL( mtFinished(long,const QString&,const QString&,int) ),
			this, SLOT( amarokSuccess(long,const QString&,const QString&) ) );
		connect( job, SIGNAL( mtError(long,const QString&,const QString&,int) ),
			this, SLOT( amarokError(long,const QString&) ) );

		if ( job->enqueue( &m_amarokWorkerThread ) != -1 )
		{
			m_logger.log( "job succesfully enqueued in amaroK's TTMD worker thread" );
			if ( ! m_fileWorkerThread.paused() )
			{
				m_logger.log( "sending pause signal to main worker thread" );
				m_fileWorkerThread.pause();
			}
		}
		else
		{
			m_logger.log( "error enqueuing job in amaroK's TTMD worker thread" );
			m_jobs.remove( job );
		}
	}
	else
	{
		m_logger.log( "message received from amaroK" );

		QRegExp profileRegExp( "customMenuClicked: transKode \\[ (.*) \\] " );
		profileRegExp.search( message );
		QString profile = profileRegExp.cap( 1 );
		ProfileOptions* options = m_config.profileOptions( profile );

		if ( !m_config.profileOptionsAreValid( options ) )
		{
			m_logger.log( "invalid profile requested, message discarted" );
			if ( options != 0 )
				delete options;
			return;
		}

		if ( m_config.generalOpt( GeneralOpt::confirm ) == "true" )
			if ( KMessageBox::No == KMessageBox::questionYesNo(
				0,
				i18n(	"<qt>Process selection with chosen profile?<br/><br/>"
						"<b>Profile</b>: %1<br/>"
						"<b>Encode Type</b>: %2<br/>"
						"<b>Switches</b>: %3</qt>" )
							.arg( profile )
							.arg( options->encodeType() )
							.arg( options->switches() ),
				"transKode" ) )
			{
				m_logger.log( "operation cancelled by user" );
				delete options;
				return;
			}

		delete options;

		message = message.mid( QString( "customMenuClicked: transKode [ " + profile + " ] ").length() );
		QStringList urlsList = QStringList::split( " ", message );

		m_logger.log( "profile: " + profile );

		for ( QStringList::Iterator it = urlsList.begin(); it != urlsList.end(); ++it )
		{
			KURL srcURL( *it );
			if ( srcURL.protocol() == "file" )
			{
				m_logger.log( "file: " + srcURL.path() );

				TransKodeFileJob* job = new TransKodeFileJob( m_config, srcURL.path() );
				job->setProfile( profile );
				m_jobs.append( job );

				connect( job, SIGNAL( mtStarted(long,const QString&,const QString&,int) ),
					this, SLOT( displayJobStarted(long,const QString&,const QString&,int) ) ) ;

				// don't connect deleteJob directly if something needs to get
				// done because we can't tell wich signal will be called first
				connect( job, SIGNAL( mtFinished(long,const QString&,const QString&,int) ),
					this, SLOT( deleteJob(long) ) );
				connect( job, SIGNAL( mtError(long,const QString&,const QString&,int) ),
					this, SLOT( displayJobError(long,const QString&,const QString&) ) );

				if ( job->enqueue( &m_fileWorkerThread ) != -1 )
					m_logger.log( "job succesfully enqueued in main worker thread" );
				else
				{
					m_logger.log( "error enqueuing job in main worker thread" );
					m_jobs.remove( job );
				}
			}
			else
			{
				//TODO: add job to stream worker (not implemented yet)
			}
		}
	}
}

/*void TransKodeScript::displayStreamRipStarted( int, int, const QString& URL )
{
	if ( m_config.generalOpt( GeneralOpt::notify ) != "true" )
		return;

	QString showMsg = i18n( "<qt><font size=\"+1\"><b>transKode: ripping started</b></font><br/><br/>"
							"<b>URL</b>: %1<br/>"
							"<b>Duration</b>: %2 seconds</qt>" )
						.arg( URL )
						.arg( m_config.generalOpt( GeneralOpt::rip_duration ) );

	displayPopup( showMsg );
}*/

/*void TransKodeScript::displayStreamRipFinished( int, int, const QString& URL, const QStringList& rippedFiles )
{
	if ( m_config.generalOpt( GeneralOpt::notify ) != "true" )
		return;

	QString showMsg = i18n(	"<font size=\"+1\"><b>transKode: done ripping %1</b></font><br/><br/>"
							"<b>Ripped files:</b>" )
						.arg( URL );

	for ( QStringList::ConstIterator it = rippedFiles.begin() ; it != rippedFiles.end(); ++it )
		showMsg += QString( "<br/>" ) + *it;

	displayPopup( "<qt>" + showMsg + "</qt>");
}*/

void TransKodeScript::displayJobStarted( long, const QString& srcPath, const QString& dstPath, int jobType )
{
	if ( m_config.generalOpt( GeneralOpt::notify ) != "true" )
		return;

	QString showMsg = "";

	if ( jobType == TransKodeFileJob::trans )
		showMsg = i18n( "<qt><font size=\"+1\"><b>transKode: transcoding started</b></font><br/><br/>"
						"<b>Source</b>: %1<br/>"
						"<b>Target</b>: %2</qt>" )
					.arg( srcPath )
					.arg( dstPath );

	/*// Don't show copy notifications since it causes notifications to overlap
	else if ( jobType == TransKodeFileJob::copy )
		showMsg = i18n( "<qt><font size=\"+1\"><b>transKode: copying file</b></font><br/><br/>"
						"<b>Source</b>: %1<br/>"
						"<b>Target</b>: %2</qt>" )
					.arg( srcPath )
					.arg( dstPath );*/

	if ( ! showMsg.isEmpty() )
		displayPopup( showMsg );
}

void TransKodeScript::displayJobError( long jobID, const QString& input, const QString& errorMsg )
{
	if ( m_config.generalOpt( GeneralOpt::notify ) != "true" )
		return;

	displayPopup( i18n( "<qt><b><font size=\"+1\" color=\"#ff0000\">transKode: %1</font><br/><br/>"
						"Source</b>: %2</qt>" )
					.arg( errorMsg )
					.arg( input ) );

	deleteJob( jobID );
}

void TransKodeScript::displayRoundStats( long,
	int transErrorCount, int transOKCount,
	int copyErrorCount, int copyOKCount,
	int jobsCancelled, int profErrorCount )
{
	if ( m_config.generalOpt( GeneralOpt::notify ) != "true" )
		return;

	QString showMsg( "" );

	if ( transErrorCount + transOKCount )
		showMsg += "<br/>" + i18n( "%1/%2 files succesfully encoded" )
			.arg( transOKCount )
			.arg( transErrorCount + transOKCount );

	if ( copyErrorCount + copyOKCount )
		showMsg += "<br/>" + i18n( "%1/%2 files succesfully copied" )
			.arg( copyOKCount )
			.arg( copyErrorCount + copyOKCount );

	if ( profErrorCount )
		showMsg += "<br/>" + i18n( "1 job with profile errors found", "%n jobs with profile errors found", profErrorCount );

	if ( jobsCancelled ) // is not possible to cancel jobs from amaroK ATM
		showMsg += "<br/>" + i18n( "1 job cancelled by user", "%n jobs cancelled by user", jobsCancelled );

	if ( ! showMsg.isEmpty() )
		displayPopup( i18n( "<qt><font size=\"+1\"><b>transKode: all done</b></font><br/>%1</qt>" ).arg( showMsg ) );
}

void TransKodeScript::displayPopup( const QString& text, int msec )
{
	KPassivePopup::message( "", text, QPixmap(), (QWidget*)0, 0, msec );
}

void TransKodeScript::amarokSuccess( long jobID, const QString& src, const QString& dst )
{
	m_logger.log( "amaroK's TTMD job completed succesfully" );

	m_logger.log( "sending success notification to amaroK's TTMD" );
	//m_logger.log( "transcodingFinished '" + KURL::fromPathOrURL( src ).url() + "' '" + KURL::fromPathOrURL( dst ).url() + "'");
	DCOPRef amarokmedia( "amarok", "mediabrowser" );
	amarokmedia.send( "transcodingFinished", KURL::fromPathOrURL( src ).url(), KURL::fromPathOrURL( dst ).url() );

	deleteJob( jobID );
}

void TransKodeScript::amarokError( long jobID, const QString& src )
{
	m_logger.log( "amaroK's TTMD job completed with error" );

	m_logger.log( "sending error notification to amaroK's TTMD" );
	//m_logger.log( "transcodingFinished '" + KURL::fromPathOrURL( src ).url() + "' ''");
	DCOPRef amarokmedia( "amarok", "mediabrowser" );
	amarokmedia.send( "transcodingFinished", KURL::fromPathOrURL( src ).url(), "" );

	deleteJob( jobID );
}

void TransKodeScript::amarokDone()
{
	m_logger.log( "amaroK's TTMD worker thread done processing" );

	m_logger.log( "sending resume signal to main worker thread" );
	m_fileWorkerThread.resume();
}

void TransKodeScript::deleteJob( long jobID )
{
	for ( TransKodeFileJob* job = m_jobs.first(); job != 0; job = m_jobs.next() )
		if ( job->id() == jobID )
		{
			m_jobs.remove();
			break;
		}
}

