/***************************************************************************
 *   Copyright (C) 2005 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "transkodefileworker.h"
#include "transkodefilejobevents.h"
#include "extratags.h"
#include <transkodefiletyperesolver.h>
#include <qprocesswrapper.h>
#include <commondefs.h>

#include <kapplication.h>
#include <kurl.h>
#include <klocale.h>

#include <qfileinfo.h>
#include <qdir.h>
#include <qregexp.h>

#define PARSE_PROCESS_OUTPUT

//NOTE: if needed, ALWAYS lock the worker jobsMutex before locking the job

// TagLib documentation doesn't say who takes cares of deleting the added
// file type resolver. ATM, WE ARE DOING IT (since it's created on the stack).
// Will create and register a file type resolver for use with TagLib
TransKodeFileTypeResolver TransKodeFileJob::s_fileTypeResolver( true );

TransKodeFileJob::TransKodeFileJob( const TransKodeConfig& cfg, const QString& srcPath, const QString& profile ):
	m_jobID( (long)this ),
	m_config( cfg ),
	m_jobMutex( true ), // recursive mutex
	m_worker( 0 ),
	m_queuePos( -1 ),
	m_cancelled( 0 ),
	m_progress( -1 ),
	m_profile( String::deepCopy( profile ) ),
	m_srcPath( QDir::cleanDirPath( srcPath ) )
{
	readTags();

	for ( int idx = 0; idx < Tag::SIZE; ++idx )
		m_userTags[idx] = "";

	m_options = m_config.profileOptions( profile );
	if ( !m_config.profileOptionsAreValid( m_options ) )
	{
		if ( m_options != 0 )
			delete m_options;
		m_options = 0;
		setDstPath( "" );
	}
	else
		setDstPath( generateDstPath() );
}

TransKodeFileJob::TransKodeFileJob( const TransKodeConfig& cfg, const QString& srcPath ):
	m_jobID( (long)this ),
	m_config( cfg ),
	m_jobMutex( true ), // recursive mutex
	m_worker( 0 ),
	m_queuePos( -1 ),
	m_cancelled( 0 ),
	m_progress( -1 ),
	m_profile( "" ),
	m_options( 0 ),
	m_srcPath( QDir::cleanDirPath( srcPath ) )
{
	readTags();

	for ( int idx = 0; idx < Tag::SIZE; ++idx )
		m_userTags[idx] = "";

	setDstPath( "" );
}

TransKodeFileJob::~TransKodeFileJob()
{
	if ( m_options != 0 )
		delete m_options;

	System::delPotentialFile( m_dstPath );
}

long TransKodeFileJob::id() const
{
	// there's no need to use a mutex with this one since its a const
	return m_jobID;
}
TransKodeFileWorker* TransKodeFileJob::worker() const
{
	m_jobMutex.lock();

	TransKodeFileWorker* worker = m_worker;

	m_jobMutex.unlock();

	return worker;
}

bool TransKodeFileJob::enqueued() const
{
	m_jobMutex.lock();

	bool ret = m_worker != 0;

	m_jobMutex.unlock();

	return ret;
}

bool TransKodeFileJob::inProgress() const
{
	m_jobMutex.lock();

	bool ret = m_queuePos == 0;

	m_jobMutex.unlock();

	return ret;
}

int TransKodeFileJob::progress() const
{
	m_jobMutex.lock();

	int ret = m_progress;

	m_jobMutex.unlock();

	return ret;
}

int TransKodeFileJob::queuePos() const
{
	m_jobMutex.lock();

	int ret = m_queuePos;

	m_jobMutex.unlock();

	return ret;
}

bool TransKodeFileJob::profileError() const
{
	m_jobMutex.lock();

	bool ret = m_options == 0;

	m_jobMutex.unlock();

	return ret;
}

QString TransKodeFileJob::profile() const
{
	m_jobMutex.lock();

	QString ret = String::deepCopy( m_profile );

	m_jobMutex.unlock();

	return ret;
}

// returns true if the job was modified
bool TransKodeFileJob::setProfile( const QString& prof )
{
	if ( inProgress() )
		return false;

	if ( profile() == prof )
		return refresh();

	m_jobMutex.lock();

	m_progress = -1;

	ProfileOptions* newOptions = m_config.profileOptions( prof );

	bool newProfileValid = m_config.profileOptionsAreValid( newOptions );
	bool emitProfileErrorChanged = newProfileValid && m_options == 0 || !newProfileValid && m_options != 0;
	bool emitDstPathChanged = false;

	m_profile = String::deepCopy( prof );

	if ( !newProfileValid )
	{
		if ( newOptions != 0)
			delete newOptions;
		m_options = 0;
		if ( ! m_dstPath.isEmpty() )
			emitDstPathChanged = true;
		setDstPath( "" );
	}
	else
	{
		m_options = newOptions;
		QString newDstPath = generateDstPath();
		if ( m_dstPath != newDstPath )
		{
			setDstPath( newDstPath );
			emitDstPathChanged = true;
		}
	}

	m_jobMutex.unlock();

	if ( emitDstPathChanged )
	{
		QString signalDstPath = dstPath();
		emit dstPathChanged( id(), signalDstPath );
		delaySignal( new DstPathChangedEvent( signalDstPath ) );
	}

	if ( emitProfileErrorChanged )
	{
		QString signalSrcPath = srcPath();
		if ( newProfileValid )
		{
			emit profileErrorUnset( id(), signalSrcPath );
			delaySignal( new ProfileErrorUnsetEvent( signalSrcPath ) );
		}
		else
		{
			emit profileErrorSet( id(), signalSrcPath );
			delaySignal( new ProfileErrorSetEvent( signalSrcPath ) );
		}
	}

	return emitDstPathChanged || emitProfileErrorChanged;
}

QString* TransKodeFileJob::tags() const
{
	m_jobMutex.lock();

	QString* ret = new QString[Tag::SIZE];
	for ( int idx = 0; idx < Tag::SIZE; ++idx )
		ret[idx] = String::deepCopy( m_userTags[idx].isEmpty() ? m_fileTags[idx] : m_userTags[idx] );

	m_jobMutex.unlock();

	return ret;
}

// returns true if the job was modified
bool TransKodeFileJob::setTags( const QString* tags )
{
	if ( inProgress() || tags == 0 )
		return false;

	m_jobMutex.lock();

	m_progress = -1;

	ProfileOptions* newOptions = m_config.profileOptions( m_profile );

	bool nowProfileValid = m_config.profileOptionsAreValid( newOptions );
	bool emitProfileErrorChanged = nowProfileValid && m_options == 0 || !nowProfileValid && m_options != 0;
	bool emitDstPathChanged = false;
	bool tagsChanged = false;

	for ( int idx = 0; idx < Tag::SIZE; ++idx )
		if ( m_userTags[idx] != tags[idx] )
		{
			m_userTags[idx] = String::deepCopy( tags[idx] );
			tagsChanged = true;
		}

	if ( !nowProfileValid )
	{
		if ( newOptions )
			delete newOptions;
		m_options = 0;
		if ( ! m_dstPath.isEmpty() )
			emitDstPathChanged = true;
		setDstPath( "" );
	}
	else
	{
		m_options = newOptions;
		QString newDstPath = generateDstPath();
		if ( m_dstPath != newDstPath )
		{
			setDstPath( newDstPath );
			emitDstPathChanged = true;
		}
	}

	m_jobMutex.unlock();

	if ( emitProfileErrorChanged )
	{
		QString signalSrcPath = srcPath();
		if ( nowProfileValid )
		{
			emit profileErrorUnset( id(), signalSrcPath );
			delaySignal( new ProfileErrorUnsetEvent( signalSrcPath ) );
		}
		else
		{
			emit profileErrorSet( id(), signalSrcPath );
			delaySignal( new ProfileErrorSetEvent( signalSrcPath ) );
		}
	}

	return emitProfileErrorChanged || emitDstPathChanged || tagsChanged;
}

bool TransKodeFileJob::refresh()
{
	if ( inProgress() )
		return false;

	m_jobMutex.lock();

	m_progress = -1;

	ProfileOptions* newOptions = m_config.profileOptions( m_profile );
	bool newProfileValid = m_config.profileOptionsAreValid( newOptions );
	bool emitProfileErrorChanged = newProfileValid && m_options == 0 || !newProfileValid && m_options != 0;
	bool emitDstPathChanged = false;

	if ( !newProfileValid )
	{
		if ( newOptions != 0 )
			delete newOptions;
		m_options = 0;
		if ( ! m_dstPath.isEmpty() )
			emitDstPathChanged = true;
		setDstPath( "" );
	}
	else
	{
		m_options = newOptions;
		QString newDstPath = generateDstPath();
		if ( m_dstPath != newDstPath )
		{
			setDstPath( newDstPath );
			emitDstPathChanged = true;
		}
	}

	m_jobMutex.unlock();

	if ( emitDstPathChanged )
	{
		QString signalDstPath = dstPath();
		emit dstPathChanged( id(), signalDstPath );
		delaySignal( new DstPathChangedEvent( signalDstPath ) );
	}

	if ( emitProfileErrorChanged )
	{
		QString signalSrcPath = srcPath();
		if ( newProfileValid )
		{
			emit profileErrorUnset( id(), signalSrcPath );
			delaySignal( new ProfileErrorUnsetEvent( signalSrcPath ) );
		}
		else
		{
			emit profileErrorSet( id(), signalSrcPath );
			delaySignal( new ProfileErrorSetEvent( signalSrcPath ) );
		}
	}

	return emitProfileErrorChanged || emitDstPathChanged;
}

QString TransKodeFileJob::srcPath() const
{
	m_jobMutex.lock();

	QString ret = String::deepCopy( m_srcPath );

	m_jobMutex.unlock();

	return ret;
}

QString TransKodeFileJob::dstPath() const
{
	m_jobMutex.lock();

	QString ret = String::deepCopy( m_dstPath );

	m_jobMutex.unlock();

	return ret;
}

void TransKodeFileJob::setDstPath( const QString& dstPath )
{
	System::delPotentialFile( m_dstPath );
	System::addPotentialFile( dstPath );

	m_dstPath = dstPath;
}

// Invoked by the worker when the job position changes on the queue
void TransKodeFileJob::advanceQueuePos()
{
	m_jobMutex.lock();

	int pos = --m_queuePos;

	m_jobMutex.unlock();

	emit posChanged( id(), pos );
	delaySignal( new PosChangedEvent( pos ) );
}

int TransKodeFileJob::enqueue( TransKodeFileWorker* w )
{
	if ( worker() != 0 || w == 0 )
		return -1;

	int queuePos;
	if ( (queuePos = w->enqueue( this )) ) // returns -1 if the job was not enqueued
	{
		m_jobMutex.lock();

		m_queuePos = queuePos;
		m_cancelled = 0;
		m_worker = w;
		m_progress = -1;

		m_jobMutex.unlock();
	}

	if ( queuePos >= 0 )
	{
		emit enqueued( id(), queuePos );
		delaySignal( new EnqueuedEvent( queuePos ) );
	}

	return queuePos;
}

// true if the job is not queued on return
bool TransKodeFileJob::tryCancel()
{
	TransKodeFileWorker* w = worker();
	if ( w == 0 )
		return true;

	bool dequeued;

	if ( dequeued = w->dequeue( this ) )
	{
		m_jobMutex.lock();

		m_cancelled = 0;
		m_queuePos = -1;
		m_worker = 0;

		m_jobMutex.unlock();
	}
	else // it wasn't in the worker queue (it can still be the current job)
	{
		m_jobMutex.lock();

		m_cancelled = 1; // set to cancel request value

		m_jobMutex.unlock();
	}

	if ( dequeued )
	{
		emit cancelled( id() );
		delaySignal( new CancelledEvent() );
	}

	return dequeued;
}

// internal use
bool TransKodeFileJob::cancelled() const
{
	m_jobMutex.lock();

	bool ret = m_cancelled == 2;

	m_jobMutex.unlock();

	return ret;
}

// internal use
bool TransKodeFileJob::cancelRequested() const
{
	m_jobMutex.lock();

	bool ret = m_cancelled == 1;

	m_jobMutex.unlock();

	return ret;
}

QString TransKodeFileJob::resolveTagForPath( Tag::TTag tagKey )
{
	m_jobMutex.lock();

	QString tagValue = m_userTags[tagKey].isEmpty() ? m_fileTags[tagKey] : m_userTags[tagKey];
	tagValue = String::deepCopy( tagValue.stripWhiteSpace() );

	m_jobMutex.unlock();

	if ( tagValue.isEmpty() )
		tagValue = m_config.tagDefault( tagKey ).stripWhiteSpace();

	return String::evaluateCase( tagValue, m_config.tagCase( tagKey ) );
}

QString TransKodeFileJob::resolveTagForFile( Tag::TTag tagKey )
{
	m_jobMutex.lock();

	QString tagValue = m_userTags[tagKey].isEmpty() ? m_fileTags[tagKey] : m_userTags[tagKey];
	tagValue = String::deepCopy( tagValue.stripWhiteSpace() );

	m_jobMutex.unlock();

	return String::evaluateCase( tagValue, m_config.tagCase( tagKey ) );
}

// call this function with the object locked
QString TransKodeFileJob::generateDstPath()
{
	if ( m_options == 0 )
		return "";

	bool overwrite = m_config.generalOpt( GeneralOpt::overwrite ) == "true";
	bool caseInsensitive = m_config.generalOpt( GeneralOpt::case_insensitive ) == "true";
	QString replaceSpecialChars = m_config.generalOpt( GeneralOpt::replace_chars );

	QFileInfo srcInfo( m_srcPath );
	QString srcName = srcInfo.baseName( true ).replace( "}", "\\}" ).replace( "{", "\\{" );
	QString srcExt = srcInfo.extension( false ).replace( "}", "\\}" ).replace( "{", "\\{" );
	QString srcDir = srcInfo.dirPath().replace( "}", "\\}" ).replace( "{", "\\{" );

	QString dstExt = m_options->bypass() ? srcExt : m_options->extension();
	dstExt.replace( "}", "\\}" ).replace( "{", "\\{" ).replace( "/", "_" );

	QString tagsForPath[Tag::SIZE];
	for ( int idx = 0; idx < Tag::SIZE; ++idx )
	{
		tagsForPath[idx] = resolveTagForPath( (Tag::TTag)idx );
		tagsForPath[idx].replace( "}", "\\}" ).replace( "{", "\\{" ).replace( "/", "_" );
	}

	QString paddedTrack = String::padNumber( tagsForPath[Tag::track], 2 );


	QString dstPath = m_options->namingScheme();

	dstPath.replace( "%{src_dir}",	srcDir );
	dstPath.replace( "%{src_name}",	srcName );
	dstPath.replace( "%{src_ext}",	srcExt );
	dstPath.replace( "%{dst_ext}",	dstExt );

	dstPath.replace( "%{artist}",	tagsForPath[Tag::artist] );
	dstPath.replace( "%{album}",	tagsForPath[Tag::album] );
	dstPath.replace( "%{year}",		tagsForPath[Tag::year] );
	dstPath.replace( "%{track#}",	tagsForPath[Tag::track] );
	dstPath.replace( "%{track##}",	paddedTrack );
	dstPath.replace( "%{title}",	tagsForPath[Tag::title] );
	dstPath.replace( "%{comment}",	tagsForPath[Tag::comment] );
	dstPath.replace( "%{genre}",	tagsForPath[Tag::genre] );
	dstPath.replace( "%{composer}",	tagsForPath[Tag::composer] );
	dstPath.replace( "%{discnum}",	tagsForPath[Tag::discnum] );

	dstPath = String::evaluateCaseExpression( dstPath ).replace( "\\}", "}" ).replace( "\\{", "{" );

	if ( ! replaceSpecialChars.isEmpty() )
		for ( unsigned int idx = 0; idx < replaceSpecialChars.length() ; ++idx )
			dstPath.replace( replaceSpecialChars[idx] , "_" );

	if ( dstPath.length() == 0 )
		return "";

	if ( dstPath[0] != '/' )
		dstPath = QDir::currentDirPath() + "/" + dstPath;

	if ( !overwrite )
		return System::nonConflictivePath( dstPath, false, caseInsensitive, true, m_dstPath );
	else
	{
		// make sure not to collide with dirs or symlinks
		return System::nonConflictivePath( dstPath, true /*onlyDir*/, caseInsensitive, false /*potentialFiles*/, QString::null );

// 		QString collidingPath = System::conflictivePath( dstPath, caseInsensitive, false, QString::null );
// 		if ( collidingPath.isEmpty() ) // there was no colliding path
// 			return dstPath;
// 		else
// 		{
// 			QFileInfo fileInfo( collidingPath );
// 			if ( fileInfo.isDir() || fileInfo.isSymLink() ) // dstPath collides with a directory or symlink
// 				return System::nonConflictivePath( collidingPath, true /*onlyDir*/, caseInsensitive, false /*potentialFiles*/, QString::null );
// 			else
// 				return collidingPath;
// 		}
	}
}

bool TransKodeFileJob::monitorProcess( QProcessWrapper& process )
{
	do
	{
		if ( cancelRequested() )
		{
			m_jobMutex.lock();

			m_cancelled = 2; // set cancelled value

			m_jobMutex.unlock();

			process.terminate( 1000 );
			break;
		}

		worker()->checkPaused( process );
	}
	while ( process.isRunning() && !process.waitProcess( 1500 ) );

	return cancelled();
}

int TransKodeFileJob::decodeToWav( const QString& srcPath, const QString& dstPath )
{
	m_worker->log( "decoding " + srcPath + " to " + dstPath );

	QString srcExt = QFileInfo( srcPath ).extension( false ).lower();

	bool stopped = false;
	int retCode = -1;

	QStringList decoderNames = TransKodePluginsManager::decoderPluginNames();
	for ( QStringList::Iterator it = decoderNames.begin(); it != decoderNames.end(); ++it )
	{
		const TransKodeDecoderPlugin* plugin = TransKodePluginsManager::decoderPlugin( *it );
		if ( plugin == 0 )
			continue;
		if ( !plugin->canDecodeExtension( srcExt ) )
			continue;

#ifdef PARSE_PROCESS_OUTPUT
		QProcessWrapper process( m_config.generalOpt( GeneralOpt::proc_priority ).toInt(), QProcess::Stdout|QProcess::Stderr|QProcess::DupStderr );
#else
		QProcessWrapper process( m_config.generalOpt( GeneralOpt::proc_priority ).toInt(), 0 );
#endif
		plugin->setupArgs( process, srcPath, dstPath );

#ifdef PARSE_PROCESS_OUTPUT
		OutputParserProxy* outputParserProxy = 0;
		ProgressParser* progressParser = plugin->newProgressParser();
		if ( progressParser )
		{
			outputParserProxy = new OutputParserProxy( &process, progressParser );
			connect( progressParser, SIGNAL( progress(int) ), this, SLOT( setDecodeProgress(int) ) );
		}
#endif

		stopped = false;
		retCode = -1;

		m_worker->log( "process args: " + process.arguments().join( ":" ) );
		m_worker->log( "decoding process started" );

		if ( process.start() )
		{
			stopped = monitorProcess( process ); // blocks until the process ends
			retCode = process.exitStatus();
		}

#ifdef PARSE_PROCESS_OUTPUT
		if ( outputParserProxy )
			delete outputParserProxy;
#endif

		if ( retCode || stopped || !QFile::exists( dstPath ) )
		{
			if ( retCode )
				m_worker->log( QString( "decoding process exited with error (%1)" ).arg( retCode ) );
			else if ( stopped )
			{
				m_worker->log( "decoding process cancelled" );
				return 1;
			}
			else
				m_worker->log( "decoding process error (output file not found)" );
		}
		else
		{
			m_worker->log( "decoding process finished successfully" );
			return 0;
		}
	}

	return 1;
}

int TransKodeFileJob::encodeFromWav( const QString& srcPath, const QString& dstPath )
{
	m_worker->log( "encoding " + srcPath + " to " + dstPath );

	QString switches = m_options->switches();

	const TransKodeEncoderPlugin* plugin = TransKodePluginsManager::encoderPlugin( m_options->encodeType() );
	if ( plugin == 0 ? true : ! plugin->isLocallySupported() )
	{
		m_worker->log( "error: encoding type (" + m_options->encodeType() + ") not supported" );
		return -1;
	}

	bool caseInsensitive = m_config.generalOpt( GeneralOpt::case_insensitive ) == "true";

	QString existingPath = System::conflictivePath( dstPath, caseInsensitive );
	QString backupPath = System::nonConflictivePath( existingPath + ".bak", false, caseInsensitive );
	if ( ! existingPath.isEmpty() ) // if dstPath file exist make a backup
	{
		m_worker->log( "backing up " + existingPath + " to " + backupPath );
		System::move( existingPath, backupPath );
	}

	QString metadata[Tag::SIZE];
	for ( int idx = 0; idx < Tag::SIZE; ++idx )
		metadata[idx] = resolveTagForFile( (Tag::TTag)idx );

#ifdef PARSE_PROCESS_OUTPUT
	QProcessWrapper process(
		m_config.generalOpt( GeneralOpt::proc_priority ).toInt(),
		QProcess::Stdout|QProcess::Stderr|QProcess::DupStderr );
#else
	QProcessWrapper process( m_config.generalOpt( GeneralOpt::proc_priority ).toInt(), 0 );
#endif

	plugin->setupArgs( process, srcPath, dstPath, switches, metadata );

#ifdef PARSE_PROCESS_OUTPUT
	OutputParserProxy* outputParserProxy = 0;
	ProgressParser* progressParser = plugin->newProgressParser();
	if ( progressParser )
	{
		outputParserProxy = new OutputParserProxy( &process, progressParser );
		connect( progressParser, SIGNAL( progress(int) ), this, SLOT( setEncodeProgress(int) ) );
	}
#endif

	m_worker->log( "process args: " + process.arguments().join( ":" ) );
	m_worker->log( "encoding process started" );

	bool stopped = false;
	int retCode = 1;

	if ( process.start() )
	{
		stopped = monitorProcess( process );
		retCode = process.exitStatus();
	}

#ifdef PARSE_PROCESS_OUTPUT
	if ( outputParserProxy )
		delete outputParserProxy;
#endif

	if ( retCode || stopped || !QFile::exists( dstPath ) )
	{
		if ( retCode )		m_worker->log( QString( "encoding process exited with error (%1)" ).arg( retCode ) );
		else if ( stopped )	m_worker->log( "encoding process cancelled" );
		else				m_worker->log( "encoding process error (output file not found)" );

		System::remove( dstPath );
		if ( ! existingPath.isEmpty() )
		{
			m_worker->log( "restoring backup from " + backupPath + " to " + existingPath );
			System::move( backupPath, existingPath );
		}

		return 1;
	}
	else
	{
		m_worker->log( "encoding process finished successfully" );

		if ( !existingPath.isEmpty() && m_config.generalOpt( GeneralOpt::overwrite ) == "true" )
		{
			m_worker->log( "removing backup" );
			System::remove( backupPath );
		}

		return 0;
	}
}

void TransKodeFileJob::readTags()
{
	s_fileTypeResolver.lock();

	s_fileTypeResolver.setEncodeType( QString::null );
	TagLib::FileRef fileRef( m_srcPath.utf8() );

	s_fileTypeResolver.unlock();

	if ( !fileRef.isNull() )
	{
		m_jobMutex.lock();

		m_fileTags[Tag::artist]		= TStringToQString( fileRef.tag()->artist() );
		m_fileTags[Tag::album]		= TStringToQString( fileRef.tag()->album() );
		m_fileTags[Tag::title]		= TStringToQString( fileRef.tag()->title() );
		m_fileTags[Tag::genre]		= TStringToQString( fileRef.tag()->genre() );
		m_fileTags[Tag::comment]	= TStringToQString( fileRef.tag()->comment() );
		m_fileTags[Tag::year]		= QString::number( fileRef.tag()->year() );
		m_fileTags[Tag::track]		= QString::number( fileRef.tag()->track() );
		m_fileTags[Tag::composer]	= TStringToQString( ExtraTags::getComposer( fileRef.file() ) );
		m_fileTags[Tag::discnum]	= QString::number( ExtraTags::getDiscNumber( fileRef.file() ) );

		m_jobMutex.unlock();
	}
}

void TransKodeFileJob::writeTags( const QString& encodeType )
{
	s_fileTypeResolver.lock();

	// If outputPath doesn't have an extension taglib will fail at determining the type (since, by default,
	// it's all it checks). This can be solved in some (most?) cases 'cause the file type can be determined
	// by looking at the encoding method that was used to generate the file.
	s_fileTypeResolver.setEncodeType( encodeType );
	TagLib::FileRef fileRef( m_dstPath.utf8() );
	s_fileTypeResolver.setEncodeType( QString::null );

	s_fileTypeResolver.unlock();

	if ( !fileRef.isNull() )
	{
		m_jobMutex.lock();

		fileRef.tag()->setArtist( QStringToTString( resolveTagForFile( Tag::artist ) ) );
		fileRef.tag()->setAlbum( QStringToTString( resolveTagForFile( Tag::album ) ) );
		fileRef.tag()->setTitle( QStringToTString( resolveTagForFile( Tag::title ) ) );
		fileRef.tag()->setGenre( QStringToTString( resolveTagForFile( Tag::genre ) ) );
		fileRef.tag()->setComment( QStringToTString( resolveTagForFile( Tag::comment ) ) );
		fileRef.tag()->setYear( resolveTagForFile( Tag::year ).toInt() );
		fileRef.tag()->setTrack( resolveTagForFile( Tag::track ).toInt() );
		ExtraTags::setComposer( fileRef.file(), QStringToTString( resolveTagForFile( Tag::composer ) ) );
		ExtraTags::setDiscNumber( fileRef.file(), resolveTagForFile( Tag::discnum ).toInt() );

		m_jobMutex.unlock();

		fileRef.save();
	}
}

void TransKodeFileJob::setDecodeProgress( int prog, int type )
{
	setProgress( (int)(prog/2), type );
}

void TransKodeFileJob::setEncodeProgress( int prog, int type )
{
	setProgress( 50 + (int)(prog/2), type );
}

void TransKodeFileJob::setProgress( int prog, int type )
{
	m_jobMutex.lock();

	m_progress = prog;

	m_jobMutex.unlock();

	emit progress( id(), prog, trans );
	delaySignal( new ProgressEvent( prog, type ) );
}

void TransKodeFileJob::normalExit( int type )
{

	QString signalSrcPath = srcPath();
	QString signalDstPath = dstPath();

	m_worker->addJobFinished( signalSrcPath, signalDstPath, type );

	m_jobMutex.lock();

	m_worker = 0;
	m_queuePos = -1;
	m_cancelled = 0;

	m_jobMutex.unlock();


	emit finished( id(), signalSrcPath, signalDstPath, type );
	delaySignal( new FinishedEvent( signalSrcPath, signalDstPath, type ) );
}

void TransKodeFileJob::abnormalExit( const QString& errorMsg, int type )
{
	m_jobMutex.lock();

	TransKodeFileWorker* w = m_worker;
	bool pE = m_options == 0;
	bool c = m_cancelled == 2;

	m_worker = 0;
	m_queuePos = -1;
	m_cancelled = 0;

	m_jobMutex.unlock();

	if ( pE )
		w->incProfileErrorCount();
	else if ( c )
	{
		w->incCancelledCount();

		emit cancelled( id() );
		delaySignal( new CancelledEvent() );
	}
	else
	{
		w->incJobErrorCount( type );

		QString signalSrcPath = srcPath();
		emit error( id(), signalSrcPath, errorMsg, type );
		delaySignal( new ErrorEvent( signalSrcPath, errorMsg, type ) );
	}
}

void TransKodeFileJob::transcode()
{
	// This ensures that the worker won't start transcoding before the enqueue operation is complete
	while ( worker() == 0 )
		QThread::sleep( 1 );

	QString safeSrcPath = String::deepCopy( srcPath() );
	QString safeDstPath = String::deepCopy( dstPath() );

	m_worker->log( "transcoding started" );
	m_worker->log( "source: " + safeSrcPath );
	m_worker->log( "target: " + safeDstPath );

	if ( m_options == 0 || ! QFileInfo( safeSrcPath ).exists() )
	{
		m_worker->log( m_options == 0 ? "error: invalid profile found" : "error: input file not found" );
		abnormalExit( "", -1 );
		return;
	}

	QString newDir = QFileInfo( safeDstPath ).dirPath();

	if ( m_options->bypass() ) // COPY
	{
		emit started( id(), safeSrcPath, safeDstPath, copy );
		delaySignal( new StartedEvent( safeSrcPath, safeDstPath, copy ) );

		setProgress( 0, copy );

		QStringList createdDirs;
		if ( !System::recursiveMakeDir( newDir, &createdDirs ) )
		{
			m_worker->log( "error creating target directory " + newDir );

			m_worker->log( "removing created directories" );
			for ( QStringList::Iterator it = createdDirs.begin(); it != createdDirs.end(); ++it )
			{
				m_worker->log( "removing directory" + *it );
				QDir().rmdir( *it );
			}

			abnormalExit( i18n( "error creating target directory" ), copy );
			return;
		}

		if ( !System::copy( safeSrcPath, safeDstPath ) )
		{
			m_worker->log( "error copying " + safeSrcPath + " to " + safeDstPath );

			abnormalExit( i18n( "error copying file" ), copy );
			return;
		}

		setProgress( 100, copy );

		writeTags( m_options->encodeType() );

		m_worker->log( "transcoding finished succesfully" );

		normalExit( copy );
	}
	else // TRANSCODE
	{
		emit started( id(), safeSrcPath, safeDstPath, trans );
		delaySignal( new StartedEvent( safeSrcPath, safeDstPath, trans ) );

		setProgress( 0, trans );

		bool caseInsensitive = m_config.generalOpt( GeneralOpt::case_insensitive ) == "true";

		// the path where the decompressed input file will be temporarily stored
		QString tempPath = System::nonConflictivePath( QDir::cleanDirPath( m_config.generalOpt( GeneralOpt::temp_dir ) +  QDir::separator() + "transkode_temp.wav" ), false, caseInsensitive );

		if ( decodeToWav( safeSrcPath, tempPath ) )
		{
			System::remove( tempPath );

			abnormalExit( i18n( "error decoding file" ), trans );
			return;
		}

		QStringList createdDirsList;
		if ( !System::recursiveMakeDir( newDir, &createdDirsList ) )
		{
			m_worker->log( "error creating dir" );

			System::remove( tempPath );

			for ( QStringList::Iterator it = createdDirsList.begin(); it != createdDirsList.end(); ++it )
				QDir().rmdir( *it );

			abnormalExit( i18n( "error creating target directory" ), trans );
			return;
		}

		setProgress( 50, trans );

		if ( encodeFromWav( tempPath, safeDstPath ) )
		{
			System::remove( tempPath );

			for ( QStringList::Iterator it = createdDirsList.begin(); it != createdDirsList.end(); ++it )
				QDir().rmdir( *it );

			abnormalExit( i18n( "error encoding file" ), trans );
			return;
		}

		System::remove( tempPath );

		setProgress( 100, trans );

		writeTags( m_options->encodeType() );

		m_worker->log( "transcoding finished succesfully" );

		normalExit( trans );
	}
}









void TransKodeFileJob::delaySignal( QCustomEvent* event )
{
	kapp->postEvent( this, event );
}

void TransKodeFileJob::customEvent( QCustomEvent* e )
{
	switch ( e->type() )
	{
		case PROFILEERRORSETEVENT:
			emit mtProfileErrorSet( id(), ((ProfileErrorSetEvent*)e)->m_srcPath );
			break;
		case PROFILEERRORUNSETEVENT:
			emit mtProfileErrorUnset( id(), ((ProfileErrorUnsetEvent*)e)->m_srcPath );
			break;

		case DSTPATHCHANGEDEVENT:
			emit mtDstPathChanged( id(), ((DstPathChangedEvent*)e)->m_dstPath );
			break;

		case ENQUEUEDEVENT:
			emit mtEnqueued( id(), ((EnqueuedEvent*)e)->m_queuePos );
			break;
		case POSCHANGEDEVENT:
			emit mtPosChanged( id(), ((PosChangedEvent*)e)->m_queuePos );
			break;
		case CANCELLEDEVENT:
			emit mtCancelled( id() );
			break;

		case STARTEDEVENT:
			emit mtStarted(
				id(),
				((StartedEvent*)e)->m_srcPath,
				((StartedEvent*)e)->m_dstPath,
				((StartedEvent*)e)->m_jobType );
			break;
		case PROGRESSEVENT:
			emit mtProgress(
				id(),
				((ProgressEvent*)e)->m_progress,
				((ProgressEvent*)e)->m_jobType );
			break;
		case ERROREVENT:
			emit mtError(
				id(),
				((ErrorEvent*)e)->m_srcPath,
				((ErrorEvent*)e)->m_errorMsg,
				((ErrorEvent*)e)->m_jobType );
			break;
		case FINISHEDEVENT:
			emit mtFinished(
				id(),
				((FinishedEvent*)e)->m_srcPath,
				((FinishedEvent*)e)->m_dstPath,
				((FinishedEvent*)e)->m_jobType );
			break;

		default:
			QObject::event( e );
			break;
	}
}

