/***************************************************************************
 *   Copyright (C) 2005 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "transkodefileworker.h"
#include "transkodefileworkerevents.h"
#include "transkodefilejobevents.h"

#include <kapplication.h>

TransKodeFileWorker::TransKodeFileWorker( TransKodeLogger& logger, const TransKodeConfig& cfg, bool autoRefresh ):
	QObject( 0, "TransKodeFileWorker" ),
	QThread(),

	m_logger( logger ),
	m_config( cfg ),
	m_workerID( (long)this ),
	m_autoRefresh( autoRefresh ),

	m_threadPaused( 0 ),
	m_pauseCondition(),
	m_threadPausedMutex( false ),

	m_threadStopped( false ),
	m_currentJob( 0 ),
	m_jobsList(),
	m_jobsMutex( true )
{
	m_jobsList.setAutoDelete( false );
}

long TransKodeFileWorker::id() const
{
	return m_workerID;
}


bool TransKodeFileWorker::processing() const
{
	m_jobsMutex.lock();

	bool ret = m_currentJob != 0;

	m_jobsMutex.unlock();

	return ret;
}


int TransKodeFileWorker::jobsQueued() const
{
	m_jobsMutex.lock();

	int ret = m_jobsList.count();

	m_jobsMutex.unlock();

	return ret;
}


bool TransKodeFileWorker::paused() const
{
	m_threadPausedMutex.lock();

	bool ret = m_threadPaused != 0;

	m_threadPausedMutex.unlock();

	return ret;
}

void TransKodeFileWorker::pause( unsigned long msecs )
{
	m_threadPausedMutex.lock();

	if ( m_threadPaused == 0 )
	{
		LOG( "pause: pause signal sent to worker", WORKER );
		m_threadPaused = msecs;
	}

	m_threadPausedMutex.unlock();
}

void TransKodeFileWorker::resume()
{
	m_threadPausedMutex.lock();

	if ( m_threadPaused )
		m_threadPaused = 0;

	m_threadPausedMutex.unlock();

	LOG( "resume: resume signal sent to worker", WORKER );
	m_pauseCondition.wakeAll();
}

void TransKodeFileWorker::checkPaused( QProcessWrapper& process )
{
	m_threadPausedMutex.lock();

	if ( m_threadPaused )
	{
		LOG( "checkPaused: attempting to pause worker subprocess", WORKER );
		if ( process.pause() )
		{
			LOG( "checkPaused: worker subprocess paused succesfully", WORKER );

			m_threadPausedMutex.unlock();

			emit workerProcessPaused( id() );
			delaySignal( new ProcessPausedEvent() );

			m_threadPausedMutex.lock();
		}
		else
			LOG( "checkPaused: worker subprocess could not be paused", WORKER );

		LOG( "checkPaused: worker paused", WORKER );

		m_pauseCondition.wait( &m_threadPausedMutex, m_threadPaused );

		m_threadPaused = 0;

		LOG( "checkPaused: worker resumed", WORKER );

		if ( process.resume() )
		{
			LOG( "checkPaused: worker subprocess resumed", WORKER );

			m_threadPausedMutex.unlock();

			emit workerProcessResumed( id() );
			delaySignal( new ProcessResumedEvent() );

			m_threadPausedMutex.lock();
		}
		else
			LOG( "checkPaused: worker subprocess could not be resumed", WORKER );
	}

	m_threadPausedMutex.unlock();
}


bool TransKodeFileWorker::stopped() const
{
	m_jobsMutex.lock();

	bool ret = m_threadStopped;

	m_jobsMutex.unlock();

	return ret;
}

void TransKodeFileWorker::stop()
{
	m_jobsMutex.lock();

	m_threadStopped = true;

	for ( QPtrListIterator<TransKodeFileJob> it( m_jobsList ); it.current() != 0; ++it )
		it.current()->tryCancel();

	if ( m_currentJob != 0 )
		m_currentJob->tryCancel();

	m_jobsMutex.unlock();
}


// the position of the job in this worker queue or -1 if the job is not queued
int TransKodeFileWorker::enqueue( TransKodeFileJob* job )
{
	if ( job == 0 )
		return -1;

	TransKodeFileWorker* w = job->worker();
	if ( w == this )
		return job->queuePos();
	else if ( w != 0 )
		return -1;

	m_jobsMutex.lock();

	int queuePos = -1;

	if ( !m_threadStopped )
	{
		m_jobsList.append( job );
		queuePos = m_jobsList.count();
	}

	m_jobsMutex.unlock();

	return queuePos;
}

// true if the was in the worker queue and is no longuer there
bool TransKodeFileWorker::dequeue( TransKodeFileJob* job )
{
	if ( job == 0 )
		return false;

	if ( job->worker() != this )
		return false;

	m_jobsMutex.lock();

	int idx = m_jobsList.findRef( job );

	if ( idx != -1 )
	{
		m_jobsList.remove(); // removes the current item (the one that was found)
		if ( m_jobsList.at( idx ) != 0 ) // will return 0 if the removed item was the last one
			for ( ; m_jobsList.current() != 0; m_jobsList.next() )
				m_jobsList.current()->advanceQueuePos();
	}

	m_jobsMutex.unlock();

	return idx != -1;
}

TransKodeFileJob* TransKodeFileWorker::getNextJob()
{
	m_jobsMutex.lock();

	TransKodeFileJob* job = m_currentJob = m_jobsList.take( 0 );
	if ( job != 0 )
	{
		if ( m_autoRefresh ) // refresh has to be done before the jobs positions reaches 0
			job->refresh();
		job->advanceQueuePos();
	}

	for ( QPtrListIterator<TransKodeFileJob> it( m_jobsList ); it.current() != 0; ++it )
		it.current()->advanceQueuePos();

	m_jobsMutex.unlock();

	return job;
}

void TransKodeFileWorker::run()
{
	LOG( "run: worker started", WORKER );

	while ( !stopped() )
	{
		LOG( "run: worker going to sleep", WORKER );

		QThread::sleep( 2 ); // sleep 2 seconds

		LOG( "run: worker awaken", WORKER );

		m_profileE = 0;
		m_jobsC = 0;
		m_transcodeE = 0;
		m_tSrcPaths.clear();
		m_tDstPaths.clear();
		m_copyE = 0;
		m_cSrcPaths.clear();
		m_cDstPaths.clear();

		bool jobsProcessed = false;
		TransKodeFileJob* job;
		while ( (job = getNextJob()) && !stopped() )
		{
			if ( !jobsProcessed )
			{
				jobsProcessed = true;
				emit roundStarted( m_workerID );
				delaySignal( new RoundStartedEvent() );
			}
			job->transcode();
		}

		if ( jobsProcessed )
		{
			emit roundFinished( m_workerID );
			delaySignal( new RoundFinishedEvent() );

			emit roundStats(
				m_workerID,
				m_transcodeE, m_tSrcPaths.size(),
				m_copyE, m_cSrcPaths.size(),
				m_jobsC, m_profileE );
			delaySignal( new RoundShortStatsEvent(
				m_transcodeE, m_tSrcPaths.size(),
				m_copyE, m_cSrcPaths.size(),
				m_jobsC, m_profileE ) );

			emit roundStats(
				m_workerID,
				m_transcodeE, m_tSrcPaths, m_tDstPaths,
				m_copyE, m_cSrcPaths, m_cDstPaths,
				m_jobsC, m_profileE );
			delaySignal( new RoundLongStatsEvent(
				m_transcodeE, m_tSrcPaths, m_tDstPaths,
				m_copyE, m_cSrcPaths, m_cDstPaths,
				m_jobsC, m_profileE ) );
		}
	}

	LOG( "run: worker finished", WORKER );
}

void TransKodeFileWorker::delaySignal( QCustomEvent* event )
{
	kapp->postEvent( this, event );
}

void TransKodeFileWorker::customEvent( QCustomEvent* e )
{
	switch ( e->type() )
	{
		case ROUNDSTARTEDEVENT:
			emit mtRoundStarted( m_workerID );
			break;

		case ROUNDFINISHEDEVENT:
			emit mtRoundFinished( m_workerID );
			break;

		case ROUNDSHORTSTATSEVENT:
			emit mtRoundStats(
				m_workerID,
				((RoundShortStatsEvent*)e)->m_transcodeE, ((RoundShortStatsEvent*)e)->m_transcodeF,
				((RoundShortStatsEvent*)e)->m_copyE, ((RoundShortStatsEvent*)e)->m_copyF,
				((RoundShortStatsEvent*)e)->m_jobsC, ((RoundShortStatsEvent*)e)->m_profileE );
			break;
		case ROUNDLONGSTATSEVENT:
			emit mtRoundStats(
				m_workerID,
				((RoundLongStatsEvent*)e)->m_transcodeE, ((RoundLongStatsEvent*)e)->m_tSrcFiles, ((RoundLongStatsEvent*)e)->m_tDstFiles,
				((RoundLongStatsEvent*)e)->m_copyE, ((RoundLongStatsEvent*)e)->m_cSrcFiles, ((RoundLongStatsEvent*)e)->m_cDstFiles,
				((RoundLongStatsEvent*)e)->m_jobsC, ((RoundLongStatsEvent*)e)->m_profileE );
			break;

		case PROCESSPAUSEDEVENT:
			emit mtWorkerProcessPaused( m_workerID );
			break;

		case PROCESSRESUMEDEVENT:
			emit mtWorkerProcessResumed( m_workerID );
			break;

		default:
			QObject::event( e );
			break;
	}
}

void TransKodeFileWorker::log( const QString& message )
{
	m_logger.log( message );
}

void TransKodeFileWorker::incProfileErrorCount()
{
	m_profileE++;
}

void TransKodeFileWorker::incCancelledCount()
{
	m_jobsC++;
}

void TransKodeFileWorker::incJobErrorCount( int type )
{
	if ( type == TransKodeFileJob::trans )
		m_transcodeE++;
	else if ( type == TransKodeFileJob::copy )
		m_copyE++;
}

void TransKodeFileWorker::addJobFinished( const QString& srcPath, const QString& dstPath, int type )
{
	if ( type == TransKodeFileJob::trans )
	{
		m_tSrcPaths.append( srcPath );
		m_tDstPaths.append( dstPath );
	}
	else if ( type == TransKodeFileJob::copy )
	{
		m_cSrcPaths.append( srcPath );
		m_cDstPaths.append( dstPath );
	}
}
