#ifndef _TRANSKODEFILEWORKER_H_
#define _TRANSKODEFILEWORKER_H_

/***************************************************************************
 *   Copyright (C) 2005 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include "transkodefilejob.h"
#include <transkodelogger.h>
#include <transkodeconfig.h>
#include <qprocesswrapper.h>

#include <qobject.h>
#include <qthread.h>
#include <qwaitcondition.h>
#include <qmutex.h>
#include <qptrlist.h>

/**
	NOTE: TransKodeFileWorker threads cannot be restarted as QThread threads can.
	This is due to the fact that control information (threadStopped) cannot be
	enforced to be (re)set prior to execution of run() (except in the constructor).
	It should be set DURING start() execution by the calling thread (prior to
	the actual thread creation) but start is not virtual so it cannot be made
	to include additional set up.
	It has to be that way so that if a thread does
		...
		worker.start();
		worker.stop();
		...
	worker is guaranteed to end at some point. Problem is that if threadStopped
	is initialized on run() (which executes on some arbitrary moment after the
	call to start) the value set by stop might get overwritten.
*/

class TransKodeFileWorker : public QObject, public QThread
{
	Q_OBJECT

	friend class TransKodeFileJob;

	public:

		TransKodeFileWorker( TransKodeLogger& logger, const TransKodeConfig& config, bool autoRefresh=false );

		long id() const;

		int jobsQueued() const;
		bool processing() const;

		void pause( unsigned long msecs = ULONG_MAX );
		void resume();
		void stop();

		bool paused() const;
		bool stopped() const;

	signals:

		/**
		 * Warning! this signals MAY NOT be emitted from the main thread (check qt docs on threads)
		 */
		void roundStarted( long workerID );
		void roundFinished( long workerID );

		void roundStats(
			long workerID,
			int tcErrorCount, int tcOKCount,
			int cpErrorCount, int cpOKCount,
			int cancelledCount, int pfErrorCount );

		void roundStats(
			long workerID,
			int tcErrorCount, const QStringList& tcOKSrcPaths, const QStringList& tcOKDstPaths,
			int cpErrorCount, const QStringList& cpOKSrcPaths, const QStringList& cpOKDstPaths,
			int cancelledCount, int pfErrorCount );

		void workerProcessPaused( long workerID );
		void workerProcessResumed( long workerID );


		/**
		 * This signals ARE emitted from the main thread
		 */
		void mtRoundStarted( long workerID );
		void mtRoundFinished( long workerID );

		void mtRoundStats(
			long workerID,
			int tcErrorCount, int tcOKCount,
			int cpErrorCount, int cpOKCount,
			int cancelledCount, int pfErrorCount );

		void mtRoundStats(
			long workerID,
			int tcErrorCount, const QStringList& tcOKSrcPaths, const QStringList& tcOKDstPaths,
			int cpErrorCount, const QStringList& cpOKSrcPaths, const QStringList& cpOKDstPaths,
			int cancelledCount, int pfErrorCount );

		void mtWorkerProcessPaused( long workerID );
		void mtWorkerProcessResumed( long workerID );

	protected:

		int  enqueue( TransKodeFileJob* job );
		bool dequeue( TransKodeFileJob* job );

		void log( const QString& message );

		void incProfileErrorCount();
		void incCancelledCount();
		void incJobErrorCount( int jobType );
		void addJobFinished( const QString& srcPath, const QString& dstPath, int type );

		TransKodeFileJob* getNextJob();
		virtual void run(); // reimplemented from QThread

		typedef QPtrList<TransKodeFileJob> TransKodeFileJobList;

		void checkPaused( QProcessWrapper& process );

		void slotConfig( bool saved );

		void delaySignal( QCustomEvent* event );
		virtual void customEvent( QCustomEvent* );

	protected:

		TransKodeLogger& m_logger;
		const TransKodeConfig& m_config;
		const long m_workerID;
		const bool m_autoRefresh;

		unsigned long m_threadPaused;
		QWaitCondition m_pauseCondition;
		mutable QMutex m_threadPausedMutex;

		bool m_threadStopped;
		TransKodeFileJob* m_currentJob;
		TransKodeFileJobList m_jobsList;
		mutable QMutex m_jobsMutex;

		// stats variables:
		int m_profileE;
		int m_jobsC;
		int m_transcodeE;
		QStringList m_tSrcPaths;
		QStringList m_tDstPaths;
		int m_copyE;
		QStringList m_cSrcPaths;
		QStringList m_cDstPaths;
};

#endif
